@@ -25,9 +25,12 @@
 #include "ephy-embed.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-type-builtins.h"
+#include "ephy-file-chooser.h"
 #include "ephy-file-helpers.h"
+#include "ephy-flatpak-utils.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
+#include "ephy-string.h"
 
 #include <errno.h>
 #include <glib/gi18n.h>
@@ -38,7 +41,6 @@ struct _EphyDownload {
 
   WebKitDownload *download;
 
-  char *destination;
   char *content_type;
 
   gboolean show_notification;
@@ -47,6 +49,7 @@ struct _EphyDownload {
   guint32 start_time;
   gboolean finished;
   GError *error;
+  GFileMonitor *file_monitor;
 };
 
 G_DEFINE_TYPE (EphyDownload, ephy_download, G_TYPE_OBJECT)
@@ -67,6 +70,7 @@ enum {
   FILENAME_SUGGESTED,
   ERROR,
   COMPLETED,
+  MOVED,
   LAST_SIGNAL
 };
 
@@ -194,7 +198,8 @@ parse_extension (const char *filename)
 }
 
 static gboolean
-set_destination_uri_for_suggested_filename (EphyDownload *download, const char *suggested_filename)
+set_destination_uri_for_suggested_filename (EphyDownload *download,
+                                            const char   *suggested_filename)
 {
   char *dest_dir;
   char *dest_name;
@@ -289,8 +294,8 @@ ephy_download_set_destination_uri (EphyDownload *download,
  * files" is set.
  **/
 void
-ephy_download_set_action (EphyDownload          *download,
-                          EphyDownloadActionType action)
+ephy_download_set_action (EphyDownload           *download,
+                          EphyDownloadActionType  action)
 {
   g_assert (EPHY_IS_DOWNLOAD (download));
 
@@ -397,8 +402,8 @@ ephy_download_succeeded (EphyDownload *download)
 }
 
 gboolean
-ephy_download_failed (EphyDownload *download,
-                      GError      **error)
+ephy_download_failed (EphyDownload  *download,
+                      GError       **error)
 {
   g_assert (EPHY_IS_DOWNLOAD (download));
 
@@ -424,9 +429,9 @@ ephy_download_failed (EphyDownload *download,
  *
  **/
 gboolean
-ephy_download_do_download_action (EphyDownload          *download,
-                                  EphyDownloadActionType action,
-                                  guint32                user_time)
+ephy_download_do_download_action (EphyDownload           *download,
+                                  EphyDownloadActionType  action,
+                                  guint32                 user_time)
 {
   GFile *destination;
   const char *destination_uri;
@@ -438,14 +443,16 @@ ephy_download_do_download_action (EphyDownload          *download,
   switch ((action ? action : download->action)) {
     case EPHY_DOWNLOAD_ACTION_BROWSE_TO:
       LOG ("ephy_download_do_download_action: browse_to");
-      ret = ephy_file_browse_to (destination, user_time);
+      /* Must not use this action type under flatpak! */
+      ret = ephy_file_browse_to (destination, user_time,
+                                 EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
       break;
     case EPHY_DOWNLOAD_ACTION_OPEN:
       LOG ("ephy_download_do_download_action: open");
-      ret = ephy_embed_shell_launch_handler (ephy_embed_shell_get_default (),
-                                             destination, NULL, user_time);
-      if (!ret)
-        ret = ephy_file_browse_to (destination, user_time);
+      ret = ephy_file_launch_handler (destination, user_time);
+      if (!ret && !ephy_is_running_inside_flatpak ())
+        ret = ephy_file_browse_to (destination, user_time,
+                                   EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
       break;
     case EPHY_DOWNLOAD_ACTION_NONE:
       LOG ("ephy_download_do_download_action: none");
@@ -472,6 +479,7 @@ ephy_download_dispose (GObject *object)
     download->download = NULL;
   }
 
+  g_clear_object (&download->file_monitor);
   g_clear_error (&download->error);
   g_clear_pointer (&download->content_type, g_free);
 
@@ -579,6 +587,19 @@ ephy_download_class_init (EphyDownloadClass *klass)
                                      NULL, NULL, NULL,
                                      G_TYPE_NONE,
                                      0);
+
+  /**
+   * EphyDownload::moved:
+   *
+   * The ::moved signal is emitted when a finished @download has been moved (or deleted).
+   **/
+  signals[MOVED] = g_signal_new ("moved",
+                                 G_OBJECT_CLASS_TYPE (object_class),
+                                 G_SIGNAL_RUN_LAST,
+                                 0,
+                                 NULL, NULL, NULL,
+                                 G_TYPE_NONE,
+                                 0);
   /**
    * EphyDownload::error:
    *
@@ -675,7 +696,7 @@ download_created_destination_cb (WebKitDownload *wk_download,
   }
 
   if (!download->content_type ||
-      (download->content_type && !g_content_type_equals (download->content_type, content_type))) {
+      !g_content_type_equals (download->content_type, content_type)) {
     g_free (download->content_type);
     download->content_type = content_type;
     g_object_notify_by_pspec (G_OBJECT (download), obj_properties[PROP_CONTENT_TYPE]);
@@ -715,10 +736,28 @@ display_download_finished_notification (WebKitDownload *download)
   }
 }
 
+static void
+download_file_monitor_changed (GFileMonitor      *monitor,
+                               GFile             *file,
+                               GFile             *other_file,
+                               GFileMonitorEvent  event_type,
+                               EphyDownload      *download)
+{
+  /* Skip messages for <file>.wkdownload */
+  if (strcmp (g_file_get_uri (file), webkit_download_get_destination (download->download)) != 0)
+    return;
+
+  if (event_type == G_FILE_MONITOR_EVENT_DELETED || event_type == G_FILE_MONITOR_EVENT_MOVED)
+    g_signal_emit (download, signals[MOVED], 0);
+}
+
 static void
 download_finished_cb (WebKitDownload *wk_download,
                       EphyDownload   *download)
 {
+  g_autoptr (GError) error = NULL;
+  g_autoptr (GFile) file = NULL;
+
   download->finished = TRUE;
 
   ephy_download_do_download_action (download, download->action, download->start_time);
@@ -727,6 +766,13 @@ download_finished_cb (WebKitDownload *wk_download,
     display_download_finished_notification (wk_download);
 
   g_signal_emit (download, signals[COMPLETED], 0);
+
+  file = g_file_new_for_uri (webkit_download_get_destination (wk_download));
+  download->file_monitor = g_file_monitor (file, G_FILE_MONITOR_NONE, NULL, &error);
+  if (!download->file_monitor)
+    g_warning ("Could not add a file monitor for %s, error: %s\n", g_file_get_uri (file), error->message);
+  else
+    g_signal_connect_object (download->file_monitor, "changed", G_CALLBACK (download_file_monitor_changed), download, 0);
 }
 
 static void
@@ -742,6 +788,123 @@ download_failed_cb (WebKitDownload *wk_download,
   g_signal_emit (download, signals[ERROR], 0, download->error);
 }
 
+static void
+filename_suggested_cb (EphyDownload *download,
+                       const char   *suggested_filename,
+                       gpointer      user_data)
+{
+  GApplication *application;
+  GtkWidget *dialog = NULL;
+  GtkWidget *message_area;
+  GtkWidget *box;
+  GtkWindow *toplevel;
+  GtkWidget *type_label;
+  GtkWidget *from_label;
+  GtkWidget *question_label;
+  GtkWidget *filechooser;
+  WebKitDownload *webkit_download;
+  WebKitURIResponse *response;
+  g_autofree gchar *sanitized_filename = NULL;
+  g_autofree gchar *type_text = NULL;
+  g_autofree gchar *from_text = NULL;
+  g_autofree gchar *content_length = NULL;
+  const gchar *content_type;
+
+  application = G_APPLICATION (ephy_embed_shell_get_default ());
+  toplevel = gtk_application_get_active_window (GTK_APPLICATION (application));
+
+  dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
+                                   GTK_DIALOG_USE_HEADER_BAR | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+                                   GTK_MESSAGE_QUESTION,
+                                   GTK_BUTTONS_NONE,
+                                   "%s",
+                                   _("Download requested"));
+  gtk_dialog_add_buttons (GTK_DIALOG (dialog), _("_Cancel"), GTK_RESPONSE_CANCEL, _("_Download"), GTK_RESPONSE_OK, NULL);
+  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", suggested_filename);
+  message_area = gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog));
+
+  webkit_download = ephy_download_get_webkit_download (download);
+  response = webkit_download_get_response (webkit_download);
+
+  box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_pack_start (GTK_BOX (message_area), box, TRUE, TRUE, 0);
+
+  /* Type */
+  content_length = g_format_size (webkit_uri_response_get_content_length (response));
+  content_type = ephy_download_get_content_type (download);
+  type_text = g_strdup_printf (_("Type: %s (%s)"), g_content_type_get_description (content_type), content_length);
+  type_label = gtk_label_new (type_text);
+  gtk_widget_set_margin_top (type_label, 12);
+  gtk_box_pack_start (GTK_BOX (box), type_label, TRUE, TRUE, 0);
+
+  /* From */
+  from_text = g_strdup_printf (_("From: %s"), ephy_string_get_host_name (webkit_uri_response_get_uri (response)));
+  from_label = gtk_label_new (from_text);
+  gtk_box_pack_start (GTK_BOX (box), from_label, TRUE, TRUE, 0);
+
+  /* Question */
+  question_label = gtk_label_new (_("Where do you want to save the file?"));
+  gtk_widget_set_margin_top (question_label, 12);
+  gtk_box_pack_start (GTK_BOX (box), question_label, TRUE, TRUE, 0);
+
+  /* File Chooser Button */
+  filechooser = gtk_file_chooser_button_new (_("Save file"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (filechooser), g_settings_get_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_LAST_DOWNLOAD_DIRECTORY));
+  gtk_box_pack_start (GTK_BOX (box), filechooser, TRUE, TRUE, 0);
+
+  gtk_widget_show_all (box);
+
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
+    g_autofree gchar *uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (filechooser));
+    g_autofree gchar *folder = g_filename_from_uri (uri, NULL, NULL);
+    g_autofree gchar *path = g_build_filename (uri, suggested_filename, NULL);
+
+    ephy_download_set_destination_uri (download, path);
+
+    webkit_download_set_allow_overwrite (webkit_download, TRUE);
+
+    ephy_downloads_manager_add_download (ephy_embed_shell_get_downloads_manager (ephy_embed_shell_get_default ()),
+                                         download);
+
+    g_settings_set_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_LAST_DOWNLOAD_DIRECTORY, folder);
+  } else {
+    ephy_download_cancel (download);
+  }
+
+  gtk_widget_destroy (dialog);
+}
+
+EphyDownload *
+ephy_download_new_internal (WebKitDownload *download)
+{
+  EphyDownload *ephy_download;
+
+  g_assert (WEBKIT_IS_DOWNLOAD (download));
+
+  ephy_download = g_object_new (EPHY_TYPE_DOWNLOAD, NULL);
+
+  g_signal_connect_object (download, "notify::response",
+                           G_CALLBACK (download_response_changed_cb),
+                           ephy_download, 0);
+  g_signal_connect_object (download, "decide-destination",
+                           G_CALLBACK (download_decide_destination_cb),
+                           ephy_download, 0);
+  g_signal_connect_object (download, "created-destination",
+                           G_CALLBACK (download_created_destination_cb),
+                           ephy_download, 0);
+  g_signal_connect_object (download, "finished",
+                           G_CALLBACK (download_finished_cb),
+                           ephy_download, 0);
+  g_signal_connect_object (download, "failed",
+                           G_CALLBACK (download_failed_cb),
+                           ephy_download, 0);
+
+  ephy_download->download = g_object_ref (download);
+  g_object_set_data (G_OBJECT (download), "ephy-download-set", GINT_TO_POINTER (TRUE));
+
+  return ephy_download;
+}
+
 /**
  * ephy_download_new:
  * @download: a #WebKitDownload to wrap
@@ -755,28 +918,13 @@ ephy_download_new (WebKitDownload *download)
 {
   EphyDownload *ephy_download;
 
-  g_assert (WEBKIT_IS_DOWNLOAD (download));
+  ephy_download = ephy_download_new_internal (download);
 
-  ephy_download = g_object_new (EPHY_TYPE_DOWNLOAD, NULL);
-
-  g_signal_connect (download, "notify::response",
-                    G_CALLBACK (download_response_changed_cb),
-                    ephy_download);
-  g_signal_connect (download, "decide-destination",
-                    G_CALLBACK (download_decide_destination_cb),
-                    ephy_download);
-  g_signal_connect (download, "created-destination",
-                    G_CALLBACK (download_created_destination_cb),
-                    ephy_download);
-  g_signal_connect (download, "finished",
-                    G_CALLBACK (download_finished_cb),
-                    ephy_download);
-  g_signal_connect (download, "failed",
-                    G_CALLBACK (download_failed_cb),
-                    ephy_download);
-
-  ephy_download->download = g_object_ref (download);
-  g_object_set_data (G_OBJECT (download), "ephy-download-set", GINT_TO_POINTER (TRUE));
+  if (!ephy_is_running_inside_flatpak () && g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ASK_ON_DOWNLOAD)) {
+    g_signal_connect (ephy_download, "filename-suggested",
+                      G_CALLBACK (filename_suggested_cb),
+                      NULL);
+  }
 
   return ephy_download;
 }
@@ -805,6 +953,21 @@ ephy_download_new_for_uri (const char *uri)
   return ephy_download;
 }
 
+EphyDownload *
+ephy_download_new_for_uri_internal (const char *uri)
+{
+  EphyDownload *ephy_download;
+  g_autoptr (WebKitDownload) download = NULL;
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
+
+  g_assert (uri != NULL);
+
+  download = webkit_web_context_download_uri (ephy_embed_shell_get_web_context (shell), uri);
+  ephy_download = ephy_download_new_internal (download);
+
+  return ephy_download;
+}
+
 void
 ephy_download_disable_desktop_notification (EphyDownload *download)
 {
