@@ -25,17 +25,16 @@
 #include "ephy-embed-utils.h"
 
 #include "ephy-about-handler.h"
+#include "ephy-prefs.h"
 #include "ephy-settings.h"
 #include "ephy-string.h"
+#include "ephy-view-source-handler.h"
 
-#include <JavaScriptCore/JavaScript.h>
 #include <glib/gi18n.h>
+#include <jsc/jsc.h>
 #include <libsoup/soup.h>
 #include <string.h>
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-
 static GRegex *non_search_regex;
 static GRegex *domain_regex;
 
@@ -118,22 +117,24 @@ ephy_embed_utils_address_has_web_scheme (const char *address)
   if (address == NULL)
     return FALSE;
 
-  colonpos = (int)((g_strstr_len (address, 11, ":")) - address);
+  colonpos = (int)((strstr (address, ":")) - address);
 
   if (colonpos < 0)
     return FALSE;
 
   has_web_scheme = !(g_ascii_strncasecmp (address, "http", colonpos) &&
                      g_ascii_strncasecmp (address, "https", colonpos) &&
-                     g_ascii_strncasecmp (address, "ftp", colonpos) &&
                      g_ascii_strncasecmp (address, "file", colonpos) &&
                      g_ascii_strncasecmp (address, "javascript", colonpos) &&
                      g_ascii_strncasecmp (address, "data", colonpos) &&
                      g_ascii_strncasecmp (address, "blob", colonpos) &&
                      g_ascii_strncasecmp (address, "about", colonpos) &&
                      g_ascii_strncasecmp (address, "ephy-about", colonpos) &&
+                     g_ascii_strncasecmp (address, "ephy-resource", colonpos) &&
+                     g_ascii_strncasecmp (address, "ephy-source", colonpos) &&
                      g_ascii_strncasecmp (address, "gopher", colonpos) &&
-                     g_ascii_strncasecmp (address, "inspector", colonpos));
+                     g_ascii_strncasecmp (address, "inspector", colonpos) &&
+                     g_ascii_strncasecmp (address, "webkit", colonpos));
 
   return has_web_scheme;
 }
@@ -141,8 +142,19 @@ ephy_embed_utils_address_has_web_scheme (const char *address)
 gboolean
 ephy_embed_utils_address_is_existing_absolute_filename (const char *address)
 {
-  return g_path_is_absolute (address) &&
-         g_file_test (address, G_FILE_TEST_EXISTS);
+  g_autofree char *real_address = NULL;
+
+  if (!strchr (address, '#')) {
+    real_address = g_strdup (address);
+  } else {
+    gint pos;
+
+    pos = g_strstr_len (address, -1, "#") - address;
+    real_address = g_strndup (address, pos);
+  }
+
+  return g_path_is_absolute (real_address) &&
+         g_file_test (real_address, G_FILE_TEST_EXISTS);
 }
 
 static gboolean
@@ -200,24 +212,43 @@ is_bang_search (const char *address)
   return FALSE;
 }
 
+static gboolean
+is_host_with_port (const char *address)
+{
+  g_auto (GStrv) split = g_strsplit (address, ":", -1);
+  gint64 port = 0;
+
+  if (g_strv_length (split) == 2)
+    port = g_ascii_strtoll (split[1], NULL, 10);
+
+  return port != 0;
+}
+
 gboolean
 ephy_embed_utils_address_is_valid (const char *address)
 {
   char *scheme;
   gboolean retval;
+  GAppInfo *info = NULL;
 
   if (!address)
     return FALSE;
 
   scheme = g_uri_parse_scheme (address);
 
-  retval = scheme ||
+  if (scheme != NULL) {
+    info = g_app_info_get_default_for_uri_scheme (scheme);
+    g_free (scheme);
+  }
+
+  retval = info ||
            ephy_embed_utils_address_is_existing_absolute_filename (address) ||
            g_regex_match (get_non_search_regex (), address, 0, NULL) ||
            is_public_domain (address) ||
-           is_bang_search (address);
+           is_bang_search (address) ||
+           is_host_with_port (address);
 
-  g_free (scheme);
+  g_clear_object (&info);
 
   return retval;
 }
@@ -242,11 +273,14 @@ ephy_embed_utils_normalize_address (const char *address)
   if (ephy_embed_utils_address_is_existing_absolute_filename (address))
     return g_strconcat ("file://", address, NULL);
 
+  if (strcmp (address, "about:gpu") == 0)
+    return g_strdup ("webkit://gpu");
+
   if (g_str_has_prefix (address, "about:") && strcmp (address, "about:blank"))
     return g_strconcat (EPHY_ABOUT_SCHEME, address + strlen ("about"), NULL);
 
   if (!ephy_embed_utils_address_has_web_scheme (address)) {
-    SoupURI *uri;
+    g_autoptr (SoupURI) uri = NULL;
 
     uri = soup_uri_new (address);
 
@@ -259,12 +293,10 @@ ephy_embed_utils_normalize_address (const char *address)
      * handler for the scheme, and since we'll fail for localhost
      * and IP, we'd fallback to loading it as a domain. */
     if (!uri ||
-        (uri && !g_strcmp0 (uri->scheme, "localhost")) ||
-        (uri && g_hostname_is_ip_address (uri->scheme)))
+        !g_strcmp0 (uri->scheme, "localhost") ||
+        g_hostname_is_ip_address (uri->scheme) ||
+        is_host_with_port (address))
       effective_address = g_strconcat ("http://", address, NULL);
-
-    if (uri)
-      soup_uri_free (uri);
   }
 
   return effective_address ? effective_address : g_strdup (address);
@@ -279,6 +311,9 @@ ephy_embed_utils_autosearch_address (const char *search_key)
   EphyEmbedShell *shell;
   EphySearchEngineManager *search_engine_manager;
 
+  if (!g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ENABLE_AUTOSEARCH))
+    return g_strdup (search_key);
+
   shell = ephy_embed_shell_get_default ();
   search_engine_manager = ephy_embed_shell_get_search_engine_manager (shell);
   address_search = ephy_search_engine_manager_get_default_search_address (search_engine_manager);
@@ -295,11 +330,35 @@ ephy_embed_utils_autosearch_address (const char *search_key)
   return effective_address;
 }
 
+static char *
+ensure_host_name_is_lowercase (const char *address)
+{
+  g_autofree gchar *host = ephy_string_get_host_name (address);
+  g_autofree gchar *lowercase_host = NULL;
+  char *ret = NULL;
+
+  if (host == NULL) {
+    return g_strdup (address);
+  }
+
+  lowercase_host = g_utf8_strdown (host, -1);
+
+  if (strcmp (host, lowercase_host) != 0) {
+    ret = ephy_string_find_and_replace (address, host, lowercase_host);
+  } else {
+    ret = g_strdup (address);
+  }
+
+  return ret;
+}
+
 char *
 ephy_embed_utils_normalize_or_autosearch_address (const char *address)
 {
-  if (ephy_embed_utils_address_is_valid (address))
-    return ephy_embed_utils_normalize_address (address);
+  g_autofree gchar *lower_case_address = ensure_host_name_is_lowercase (address);
+
+  if (ephy_embed_utils_address_is_valid (lower_case_address))
+    return ephy_embed_utils_normalize_address (lower_case_address);
   else
     return ephy_embed_utils_autosearch_address (address);
 }
@@ -335,6 +394,9 @@ ephy_embed_utils_is_no_show_address (const char *address)
     if (!strcmp (address, do_not_show_address[i]))
       return TRUE;
 
+  if (strstr (address, EPHY_VIEW_SOURCE_SCHEME) == address)
+    return TRUE;
+
   return FALSE;
 }
 
@@ -351,65 +413,9 @@ ephy_embed_utils_get_title_from_address (const char *address)
   return ephy_string_get_host_name (address);
 }
 
-gboolean
-ephy_embed_utils_urls_have_same_origin (const char *a_url,
-                                        const char *b_url)
-{
-  SoupURI *a_uri, *b_uri;
-  gboolean retval = FALSE;
-
-  a_uri = soup_uri_new (a_url);
-  if (!a_uri)
-    return retval;
-
-  b_uri = soup_uri_new (b_url);
-  if (b_uri) {
-    retval = a_uri->host && b_uri->host && soup_uri_host_equal (a_uri, b_uri);
-    soup_uri_free (b_uri);
-  }
-
-  soup_uri_free (a_uri);
-
-  return retval;
-}
-
-char *
-ephy_embed_utils_get_js_result_as_string (WebKitJavascriptResult *js_result)
-{
-  JSValueRef js_value;
-  JSStringRef js_string;
-  size_t max_size;
-  char *retval = NULL;
-
-  js_value = webkit_javascript_result_get_value (js_result);
-  js_string = JSValueToStringCopy (webkit_javascript_result_get_global_context (js_result),
-                                   js_value, NULL);
-  max_size = JSStringGetMaximumUTF8CStringSize (js_string);
-  if (max_size) {
-    retval = g_malloc (max_size);
-    JSStringGetUTF8CString (js_string, retval, max_size);
-  }
-  JSStringRelease (js_string);
-
-  return retval;
-}
-
-double
-ephy_embed_utils_get_js_result_as_number (WebKitJavascriptResult *js_result)
-{
-  JSValueRef js_value;
-
-  js_value = webkit_javascript_result_get_value (js_result);
-
-  return JSValueToNumber (webkit_javascript_result_get_global_context (js_result),
-                          js_value, NULL);
-}
-
 void
 ephy_embed_utils_shutdown (void)
 {
-  g_clear_pointer (&non_search_regex, (GDestroyNotify)g_regex_unref);
-  g_clear_pointer (&domain_regex, (GDestroyNotify)g_regex_unref);
+  g_clear_pointer (&non_search_regex, g_regex_unref);
+  g_clear_pointer (&domain_regex, g_regex_unref);
 }
-
-#pragma GCC diagnostic pop
