@@ -26,7 +26,6 @@
 #include "ephy-embed-shell.h"
 #include "ephy-history-types.h"
 #include "ephy-security-levels.h"
-#include "ephy-web-extension-proxy.h"
 
 G_BEGIN_DECLS
 
@@ -42,7 +41,9 @@ G_DECLARE_FINAL_TYPE (EphyWebView, ephy_web_view, EPHY, WEB_VIEW, WebKitWebView)
                                         "^about:.*$|" \
                                         "^data:.*$|" \
                                         "^file:.*$|" \
-                                        "^inspector://.*$" \
+                                        "^inspector://.*$|" \
+                                        "^webkit://.*$|" \
+                                        "^ephy-resource://.*$" \
                                         ")"
 
 #define EPHY_WEB_VIEW_DOMAIN_REGEX "^localhost(\\.[^[:space:]]+)?(:\\d+)?(:[0-9]+)?(/.*)?$|" \
@@ -68,7 +69,8 @@ typedef enum {
   EPHY_WEB_VIEW_ERROR_PAGE_CRASH,
   EPHY_WEB_VIEW_ERROR_PROCESS_CRASH,
   EPHY_WEB_VIEW_ERROR_INVALID_TLS_CERTIFICATE,
-  EPHY_WEB_VIEW_ERROR_UNSAFE_BROWSING
+  EPHY_WEB_VIEW_ERROR_UNSAFE_BROWSING,
+  EPHY_WEB_VIEW_ERROR_NO_SUCH_FILE,
 } EphyWebViewErrorPage;
 
 typedef void (* EphyPasswordSaveRequestCallback) (int      response_id,
@@ -145,7 +147,7 @@ char                      *ephy_web_view_get_web_app_title_finish (EphyWebView
 void                       ephy_web_view_set_visit_type           (EphyWebView *view, 
                                                                    EphyHistoryPageVisitType visit_type);
 EphyHistoryPageVisitType   ephy_web_view_get_visit_type           (EphyWebView *view);
-void                       ephy_web_view_popups_manager_reset     (EphyWebView               *view);
+
 void                       ephy_web_view_save                     (EphyWebView               *view,
                                                                    const char                *uri);
 void                       ephy_web_view_load_homepage            (EphyWebView               *view);
@@ -162,7 +164,7 @@ gboolean                   ephy_web_view_is_reader_mode_available (EphyWebView
 
 gboolean                   ephy_web_view_get_reader_mode_state    (EphyWebView               *view);
 
-EphyWebExtensionProxy     *ephy_web_view_get_web_extension_proxy  (EphyWebView               *view);
+gboolean                   ephy_web_view_is_in_auth_dialog        (EphyWebView               *view);
 
 void                       ephy_web_view_show_auth_form_save_request (EphyWebView                    *web_view,
                                                                       const char                     *origin,
@@ -171,4 +173,6 @@ void                       ephy_web_view_show_auth_form_save_request (EphyWebVie
                                                                       gpointer                        response_data,
                                                                       GDestroyNotify                  response_destroy);
 
+gboolean                   ephy_web_view_in_pdf_viewer             (EphyWebView              *view);
+
 G_END_DECLS
