@@ -31,13 +31,6 @@ extern GQuark ephy_file_helpers_error_quark;
 
 G_BEGIN_DECLS
 
-typedef enum
-{
-  EPHY_MIME_PERMISSION_SAFE    = 1,
-  EPHY_MIME_PERMISSION_UNSAFE  = 2,
-  EPHY_MIME_PERMISSION_UNKNOWN = 3
-} EphyMimePermission;
-
 typedef enum
 {
   EPHY_FILE_HELPERS_NONE             = 0,
