@@ -31,8 +31,7 @@
 #include <string.h>
 #include <webkit2/webkit2.h>
 
-struct _EphyPermissionsManager
-{
+struct _EphyPermissionsManager {
   GObject parent_instance;
 
   GHashTable *origins_mapping;
@@ -40,6 +39,8 @@ struct _EphyPermissionsManager
 
   GHashTable *permission_type_permitted_origins;
   GHashTable *permission_type_denied_origins;
+
+  GSettingsBackend *backend;
 };
 
 G_DEFINE_TYPE (EphyPermissionsManager, ephy_permissions_manager, G_TYPE_OBJECT)
@@ -49,6 +50,8 @@ G_DEFINE_TYPE (EphyPermissionsManager, ephy_permissions_manager, G_TYPE_OBJECT)
 static void
 ephy_permissions_manager_init (EphyPermissionsManager *manager)
 {
+  g_autofree char *filename = NULL;
+
   manager->origins_mapping = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
   manager->settings_mapping = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_free);
 
@@ -56,6 +59,9 @@ ephy_permissions_manager_init (EphyPermissionsManager *manager)
    * the GList keys without destroying the contents of the lists. */
   manager->permission_type_permitted_origins = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, NULL);
   manager->permission_type_denied_origins = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, NULL);
+
+  filename = g_build_filename (ephy_profile_dir (), PERMISSIONS_FILENAME, NULL);
+  manager->backend = g_keyfile_settings_backend_new (filename, "/", NULL);
 }
 
 static void
@@ -86,6 +92,8 @@ ephy_permissions_manager_dispose (GObject *object)
     manager->permission_type_denied_origins = NULL;
   }
 
+  g_clear_object (&manager->backend);
+
   G_OBJECT_CLASS (ephy_permissions_manager_parent_class)->dispose (object);
 }
 
@@ -103,8 +111,6 @@ ephy_permissions_manager_get_settings_for_origin (EphyPermissionsManager *manage
 {
   char *origin_path;
   char *trimmed_protocol;
-  char *filename;
-  GSettingsBackend* backend;
   GSettings *settings;
   WebKitSecurityOrigin *security_origin;
   char *pos;
@@ -115,10 +121,6 @@ ephy_permissions_manager_get_settings_for_origin (EphyPermissionsManager *manage
   if (settings)
     return settings;
 
-  filename = g_build_filename (ephy_profile_dir (), PERMISSIONS_FILENAME, NULL);
-  backend = g_keyfile_settings_backend_new (filename, "/", NULL);
-  g_free (filename);
-
   /* Cannot contain consecutive slashes in GSettings path... */
   security_origin = webkit_security_origin_new_for_uri (origin);
   trimmed_protocol = g_strdup (webkit_security_origin_get_protocol (security_origin));
@@ -131,10 +133,9 @@ ephy_permissions_manager_get_settings_for_origin (EphyPermissionsManager *manage
                                  webkit_security_origin_get_host (security_origin),
                                  webkit_security_origin_get_port (security_origin));
 
-  settings = g_settings_new_with_backend_and_path ("org.gnome.Epiphany.permissions", backend, origin_path);
+  settings = g_settings_new_with_backend_and_path ("org.gnome.Epiphany.permissions", manager->backend, origin_path);
   g_free (trimmed_protocol);
   g_free (origin_path);
-  g_object_unref (backend);
   webkit_security_origin_unref (security_origin);
 
   /* Note that settings is owned only by the first hash table! */
@@ -154,18 +155,20 @@ static const char *
 permission_type_to_string (EphyPermissionType type)
 {
   switch (type) {
-  case EPHY_PERMISSION_TYPE_SHOW_NOTIFICATIONS:
-    return "notifications-permission";
-  case EPHY_PERMISSION_TYPE_SAVE_PASSWORD:
-    return "save-password-permission";
-  case EPHY_PERMISSION_TYPE_ACCESS_LOCATION:
-    return "geolocation-permission";
-  case EPHY_PERMISSION_TYPE_ACCESS_MICROPHONE:
-    return "audio-device-permission";
-  case EPHY_PERMISSION_TYPE_ACCESS_WEBCAM:
-    return "video-device-permission";
-  default:
-    g_assert_not_reached ();
+    case EPHY_PERMISSION_TYPE_SHOW_NOTIFICATIONS:
+      return "notifications-permission";
+    case EPHY_PERMISSION_TYPE_SAVE_PASSWORD:
+      return "save-password-permission";
+    case EPHY_PERMISSION_TYPE_ACCESS_LOCATION:
+      return "geolocation-permission";
+    case EPHY_PERMISSION_TYPE_ACCESS_MICROPHONE:
+      return "audio-device-permission";
+    case EPHY_PERMISSION_TYPE_ACCESS_WEBCAM:
+      return "video-device-permission";
+    case EPHY_PERMISSION_TYPE_SHOW_ADS:
+      return "advertisement-permission";
+    default:
+      g_assert_not_reached ();
   }
 }
 
@@ -294,23 +297,21 @@ origin_for_keyfile_key (GKeyFile           *file,
                         gboolean            permit)
 {
   WebKitSecurityOrigin *origin = NULL;
-  char *value;
-  GError *error = NULL;
 
   if (strcmp (permission_type_to_string (type), key) == 0) {
+    g_autoptr (GError) error = NULL;
+    g_autofree char *value = NULL;
+
     value = g_key_file_get_string (file, group, key, &error);
     if (error != NULL) {
       g_warning ("Error processing %s group %s key %s: %s",
                  filename, group, key, error->message);
-      g_error_free (error);
       return NULL;
     }
 
     if ((permit && strcmp (value, "'allow'") == 0) ||
         (!permit && strcmp (value, "'deny'") == 0))
       origin = group_name_to_security_origin (group);
-
-    g_free (value);
   }
 
   return origin;
@@ -323,27 +324,25 @@ origins_for_keyfile_group (GKeyFile           *file,
                            EphyPermissionType  type,
                            gboolean            permit)
 {
-  char **keys;
+  g_auto (GStrv) keys = NULL;
   gsize keys_length;
   GList *origins = NULL;
-  WebKitSecurityOrigin *origin;
-  GError *error = NULL;
+  g_autoptr (GError) error = NULL;
 
   keys = g_key_file_get_keys (file, group, &keys_length, &error);
   if (error != NULL) {
     g_warning ("Error processing %s group %s: %s", filename, group, error->message);
-    g_error_free (error);
     return NULL;
   }
 
   for (guint i = 0; i < keys_length; i++) {
+    WebKitSecurityOrigin *origin;
+
     origin = origin_for_keyfile_key (file, filename, group, keys[i], type, permit);
     if (origin)
       origins = g_list_prepend (origins, origin);
   }
 
-  g_strfreev (keys);
-
   return origins;
 }
 
@@ -475,6 +474,9 @@ ephy_permissions_manager_export_to_js_context (EphyPermissionsManager *manager,
   js_enum_value = jsc_value_new_number (js_context, EPHY_PERMISSION_TYPE_ACCESS_WEBCAM);
   jsc_value_object_set_property (js_enum, "ACCESS_WEBCAM", js_enum_value);
   g_object_unref (js_enum_value);
+  js_enum_value = jsc_value_new_number (js_context, EPHY_PERMISSION_TYPE_SHOW_ADS);
+  jsc_value_object_set_property (js_enum, "SHOW_ADS", js_enum_value);
+  g_object_unref (js_enum_value);
   jsc_value_object_set_property (js_namespace, "PermissionType", js_enum);
   g_object_unref (js_enum);
 }
