@@ -546,7 +546,7 @@ ephy_web_application_for_profile_directory (const char *profile_dir)
 
   id = get_app_id_from_profile_directory (profile_dir);
   if (!id)
-    g_error ("Cannot create EphyWebApplication: failed to get app ID from profile directory %s", profile_dir);
+    return NULL;
 
   app = g_new0 (EphyWebApplication, 1);
   app->id = g_strdup (id);
@@ -554,8 +554,11 @@ ephy_web_application_for_profile_directory (const char *profile_dir)
   app->desktop_file = get_app_desktop_filename (id);
   desktop_file_path = g_build_filename (profile_dir, app->desktop_file, NULL);
   desktop_info = g_desktop_app_info_new_from_filename (desktop_file_path);
-  if (!desktop_info)
-    g_error ("Cannot create EphyWebApplication: failed to create GDesktopAppInfo from desktop file %s", desktop_file_path);
+  if (!desktop_info) {
+    ephy_web_application_free (app);
+    g_free (desktop_file_path);
+    return NULL;
+  }
 
   app->name = g_strdup (g_app_info_get_name (G_APP_INFO (desktop_info)));
   app->icon_url = g_desktop_app_info_get_string (desktop_info, "Icon");
@@ -783,6 +786,8 @@ ephy_web_application_is_uri_allowed (const char *uri)
   guint i;
   gboolean matched = FALSE;
 
+  g_assert (webapp);
+
   if (g_str_has_prefix (uri, "blob:") || g_str_has_prefix (uri, "data:"))
     return TRUE;
 
