@@ -24,6 +24,8 @@
 #include "ephy-history-service-private.h"
 #include "ephy-history-types.h"
 #include "ephy-lib-type-builtins.h"
+#include "ephy-prefs.h"
+#include "ephy-settings.h"
 #include "ephy-sqlite-connection.h"
 #include "ephy-sync-utils.h"
 
@@ -31,7 +33,9 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 
-typedef gboolean (*EphyHistoryServiceMethod)      (EphyHistoryService *self, gpointer data, gpointer *result);
+typedef gboolean (*EphyHistoryServiceMethod)      (EphyHistoryService *self,
+                                                   gpointer            data,
+                                                   gpointer           *result);
 
 typedef enum {
   /* WRITE */
@@ -79,14 +83,19 @@ typedef struct _EphyHistoryServiceMessage {
 } EphyHistoryServiceMessage;
 
 static gpointer run_history_service_thread (EphyHistoryService *self);
-static void ephy_history_service_process_message (EphyHistoryService *self, EphyHistoryServiceMessage *message);
-static gboolean ephy_history_service_execute_quit (EphyHistoryService *self, gpointer data, gpointer *result);
-static void ephy_history_service_quit (EphyHistoryService *self, EphyHistoryJobCallback callback, gpointer user_data);
+static void ephy_history_service_process_message (EphyHistoryService        *self,
+                                                  EphyHistoryServiceMessage *message);
+static gboolean ephy_history_service_execute_quit (EphyHistoryService *self,
+                                                   gpointer            data,
+                                                   gpointer           *result);
+static void ephy_history_service_quit (EphyHistoryService    *self,
+                                       EphyHistoryJobCallback callback,
+                                       gpointer               user_data);
 
 enum {
   PROP_0,
   PROP_HISTORY_FILENAME,
-  PROP_READ_ONLY,
+  PROP_MEMORY,
   LAST_PROP
 };
 
@@ -95,7 +104,10 @@ static GParamSpec *obj_properties[LAST_PROP];
 G_DEFINE_TYPE (EphyHistoryService, ephy_history_service, G_TYPE_OBJECT);
 
 static void
-ephy_history_service_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
+ephy_history_service_set_property (GObject      *object,
+                                   guint         property_id,
+                                   const GValue *value,
+                                   GParamSpec   *pspec)
 {
   EphyHistoryService *self = EPHY_HISTORY_SERVICE (object);
 
@@ -104,8 +116,8 @@ ephy_history_service_set_property (GObject *object, guint property_id, const GVa
       g_free (self->history_filename);
       self->history_filename = g_value_dup_string (value);
       break;
-    case PROP_READ_ONLY:
-      self->read_only = g_value_get_boolean (value);
+    case PROP_MEMORY:
+      self->in_memory = g_value_get_boolean (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
@@ -114,7 +126,10 @@ ephy_history_service_set_property (GObject *object, guint property_id, const GVa
 }
 
 static void
-ephy_history_service_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
+ephy_history_service_get_property (GObject    *object,
+                                   guint       property_id,
+                                   GValue     *value,
+                                   GParamSpec *pspec)
 {
   EphyHistoryService *self = EPHY_HISTORY_SERVICE (object);
   switch (property_id) {
@@ -147,10 +162,7 @@ ephy_history_service_dispose (GObject *object)
 {
   EphyHistoryService *self = EPHY_HISTORY_SERVICE (object);
 
-  if (self->queue_urls_visited_id) {
-    g_source_remove (self->queue_urls_visited_id);
-    self->queue_urls_visited_id = 0;
-  }
+  g_clear_handle_id (&self->queue_urls_visited_id, g_source_remove);
 
   G_OBJECT_CLASS (ephy_history_service_parent_class)->dispose (object);
 }
@@ -285,10 +297,10 @@ ephy_history_service_class_init (EphyHistoryServiceClass *klass)
                          NULL,
                          G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
-  obj_properties[PROP_READ_ONLY] =
-    g_param_spec_boolean ("read-only",
-                          "Read only mode",
-                          "Whether the history service works in read only mode",
+  obj_properties[PROP_MEMORY] =
+    g_param_spec_boolean ("memory",
+                          "In memory mode",
+                          "Whether the history service works in memory mode",
                           FALSE,
                           G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
 
@@ -306,24 +318,26 @@ ephy_history_service_new (const char               *history_filename,
 {
   return EPHY_HISTORY_SERVICE (g_object_new (EPHY_TYPE_HISTORY_SERVICE,
                                              "history-filename", history_filename,
-                                             "read-only", mode == EPHY_SQLITE_CONNECTION_MODE_READ_ONLY,
+                                             "memory", mode == EPHY_SQLITE_CONNECTION_MODE_MEMORY,
                                              NULL));
 }
 
 static gint
-sort_messages (EphyHistoryServiceMessage *a, EphyHistoryServiceMessage *b, gpointer user_data)
+sort_messages (EphyHistoryServiceMessage *a,
+               EphyHistoryServiceMessage *b,
+               gpointer                   user_data)
 {
   return a->type > b->type ? 1 : a->type == b->type ? 0 : -1;
 }
 
 static EphyHistoryServiceMessage *
-ephy_history_service_message_new (EphyHistoryService           *service,
-                                  EphyHistoryServiceMessageType type,
-                                  gpointer                      method_argument,
-                                  GDestroyNotify                method_argument_cleanup,
-                                  GCancellable                 *cancellable,
-                                  EphyHistoryJobCallback        callback,
-                                  gpointer                      user_data)
+ephy_history_service_message_new (EphyHistoryService            *service,
+                                  EphyHistoryServiceMessageType  type,
+                                  gpointer                       method_argument,
+                                  GDestroyNotify                 method_argument_cleanup,
+                                  GCancellable                  *cancellable,
+                                  EphyHistoryJobCallback         callback,
+                                  gpointer                       user_data)
 {
   EphyHistoryServiceMessage *message = g_new0 (EphyHistoryServiceMessage, 1);
 
@@ -351,7 +365,8 @@ ephy_history_service_message_free (EphyHistoryServiceMessage *message)
 }
 
 static void
-ephy_history_service_send_message (EphyHistoryService *self, EphyHistoryServiceMessage *message)
+ephy_history_service_send_message (EphyHistoryService        *self,
+                                   EphyHistoryServiceMessage *message)
 {
   g_async_queue_push_sorted (self->queue, message, (GCompareDataFunc)sort_messages, NULL);
 }
@@ -362,8 +377,7 @@ ephy_history_service_open_transaction (EphyHistoryService *self)
   GError *error = NULL;
   g_assert (self->history_thread == g_thread_self ());
 
-  if (self->history_database == NULL ||
-      self->read_only)
+  if (self->history_database == NULL)
     return;
 
   ephy_sqlite_connection_begin_transaction (self->history_database, &error);
@@ -379,8 +393,7 @@ ephy_history_service_commit_transaction (EphyHistoryService *self)
   GError *error = NULL;
   g_assert (self->history_thread == g_thread_self ());
 
-  if (self->history_database == NULL ||
-      self->read_only)
+  if (self->history_database == NULL)
     return;
 
   ephy_sqlite_connection_commit_transaction (self->history_database, &error);
@@ -400,7 +413,7 @@ ephy_history_service_open_database_connections (EphyHistoryService *self)
   if (self->history_database != NULL)
     g_object_unref (self->history_database);
 
-  self->history_database = ephy_sqlite_connection_new (self->read_only ? EPHY_SQLITE_CONNECTION_MODE_READ_ONLY
+  self->history_database = ephy_sqlite_connection_new (self->in_memory ? EPHY_SQLITE_CONNECTION_MODE_MEMORY
                                                                        : EPHY_SQLITE_CONNECTION_MODE_READWRITE,
                                                        self->history_filename);
   ephy_sqlite_connection_open (self->history_database, &error);
@@ -408,10 +421,7 @@ ephy_history_service_open_database_connections (EphyHistoryService *self)
     g_object_unref (self->history_database);
     self->history_database = NULL;
 
-    /* Opening the database is expected to fail if it's being opened in read-
-     * only mode and does not already exist. Otherwise, this is bad. */
-    if (!self->read_only ||
-        !g_error_matches (error, EPHY_SQLITE_ERROR, SQLITE_CANTOPEN) ||
+    if (!g_error_matches (error, EPHY_SQLITE_ERROR, SQLITE_CANTOPEN) ||
         g_file_test (self->history_filename, G_FILE_TEST_EXISTS)) {
       g_warning ("Could not open history database at %s: %s", self->history_filename, error->message);
     }
@@ -421,10 +431,9 @@ ephy_history_service_open_database_connections (EphyHistoryService *self)
     ephy_sqlite_connection_enable_foreign_keys (self->history_database);
   }
 
-  return self->read_only ||
-          (ephy_history_service_initialize_hosts_table (self) &&
-           ephy_history_service_initialize_urls_table (self) &&
-           ephy_history_service_initialize_visits_table (self));
+  return (ephy_history_service_initialize_hosts_table (self) &&
+          ephy_history_service_initialize_urls_table (self) &&
+          ephy_history_service_initialize_visits_table (self));
 }
 
 static void
@@ -438,7 +447,9 @@ ephy_history_service_close_database_connections (EphyHistoryService *self)
 }
 
 static gboolean
-ephy_history_service_execute_quit (EphyHistoryService *self, gpointer data, gpointer *result)
+ephy_history_service_execute_quit (EphyHistoryService *self,
+                                   gpointer            data,
+                                   gpointer           *result)
 {
   g_assert (self->history_thread == g_thread_self ());
 
@@ -541,7 +552,8 @@ signal_emission_context_new (EphyHistoryService *service,
 }
 
 static gboolean
-ephy_history_service_execute_add_visit_helper (EphyHistoryService *self, EphyHistoryPageVisit *visit)
+ephy_history_service_execute_add_visit_helper (EphyHistoryService   *self,
+                                               EphyHistoryPageVisit *visit)
 {
   if (visit->url->host == NULL)
     visit->url->host = ephy_history_service_get_host_row_from_url (self, visit->url->url);
@@ -570,7 +582,7 @@ ephy_history_service_execute_add_visit_helper (EphyHistoryService *self, EphyHis
 
     ephy_history_service_add_url_row (self, visit->url);
 
-    if (visit->url->id == -1) {
+    if (!self->in_memory && visit->url->id == -1) {
       g_warning ("Adding visit failed after failed URL addition.");
       return FALSE;
     }
@@ -594,27 +606,25 @@ ephy_history_service_execute_add_visit_helper (EphyHistoryService *self, EphyHis
 }
 
 static gboolean
-ephy_history_service_execute_add_visit (EphyHistoryService *self, EphyHistoryPageVisit *visit, gpointer *result)
+ephy_history_service_execute_add_visit (EphyHistoryService   *self,
+                                        EphyHistoryPageVisit *visit,
+                                        gpointer             *result)
 {
   gboolean success;
   g_assert (self->history_thread == g_thread_self ());
 
-  if (self->read_only)
-    return FALSE;
-
   success = ephy_history_service_execute_add_visit_helper (self, visit);
   return success;
 }
 
 static gboolean
-ephy_history_service_execute_add_visits (EphyHistoryService *self, GList *visits, gpointer *result)
+ephy_history_service_execute_add_visits (EphyHistoryService *self,
+                                         GList              *visits,
+                                         gpointer           *result)
 {
   gboolean success = TRUE;
   g_assert (self->history_thread == g_thread_self ());
 
-  if (self->read_only)
-    return FALSE;
-
   while (visits) {
     success = success && ephy_history_service_execute_add_visit_helper (self, (EphyHistoryPageVisit *)visits->data);
     visits = visits->next;
@@ -624,7 +634,9 @@ ephy_history_service_execute_add_visits (EphyHistoryService *self, GList *visits
 }
 
 static gboolean
-ephy_history_service_execute_find_visits (EphyHistoryService *self, EphyHistoryQuery *query, gpointer *result)
+ephy_history_service_execute_find_visits (EphyHistoryService *self,
+                                          EphyHistoryQuery   *query,
+                                          gpointer           *result)
 {
   GList *visits = ephy_history_service_find_visit_rows (self, query);
   GList *current = visits;
@@ -647,7 +659,8 @@ ephy_history_service_execute_find_visits (EphyHistoryService *self, EphyHistoryQ
 
 static gboolean
 ephy_history_service_execute_get_hosts (EphyHistoryService *self,
-                                        gpointer pointer, gpointer *results)
+                                        gpointer            pointer,
+                                        gpointer           *results)
 {
   GList *hosts;
 
@@ -659,7 +672,8 @@ ephy_history_service_execute_get_hosts (EphyHistoryService *self,
 
 static gboolean
 ephy_history_service_execute_query_hosts (EphyHistoryService *self,
-                                          EphyHistoryQuery *query, gpointer *results)
+                                          EphyHistoryQuery   *query,
+                                          gpointer           *results)
 {
   GList *hosts;
 
@@ -670,7 +684,11 @@ ephy_history_service_execute_query_hosts (EphyHistoryService *self,
 }
 
 void
-ephy_history_service_add_visit (EphyHistoryService *self, EphyHistoryPageVisit *visit, GCancellable *cancellable, EphyHistoryJobCallback callback, gpointer user_data)
+ephy_history_service_add_visit (EphyHistoryService     *self,
+                                EphyHistoryPageVisit   *visit,
+                                GCancellable           *cancellable,
+                                EphyHistoryJobCallback  callback,
+                                gpointer                user_data)
 {
   EphyHistoryServiceMessage *message;
 
@@ -685,7 +703,11 @@ ephy_history_service_add_visit (EphyHistoryService *self, EphyHistoryPageVisit *
 }
 
 void
-ephy_history_service_add_visits (EphyHistoryService *self, GList *visits, GCancellable *cancellable, EphyHistoryJobCallback callback, gpointer user_data)
+ephy_history_service_add_visits (EphyHistoryService     *self,
+                                 GList                  *visits,
+                                 GCancellable           *cancellable,
+                                 EphyHistoryJobCallback  callback,
+                                 gpointer                user_data)
 {
   EphyHistoryServiceMessage *message;
 
@@ -700,7 +722,12 @@ ephy_history_service_add_visits (EphyHistoryService *self, GList *visits, GCance
 }
 
 void
-ephy_history_service_find_visits_in_time (EphyHistoryService *self, gint64 from, gint64 to, GCancellable *cancellable, EphyHistoryJobCallback callback, gpointer user_data)
+ephy_history_service_find_visits_in_time (EphyHistoryService     *self,
+                                          gint64                  from,
+                                          gint64                  to,
+                                          GCancellable           *cancellable,
+                                          EphyHistoryJobCallback  callback,
+                                          gpointer                user_data)
 {
   EphyHistoryQuery *query;
 
@@ -715,7 +742,11 @@ ephy_history_service_find_visits_in_time (EphyHistoryService *self, gint64 from,
 }
 
 void
-ephy_history_service_query_visits (EphyHistoryService *self, EphyHistoryQuery *query, GCancellable *cancellable, EphyHistoryJobCallback callback, gpointer user_data)
+ephy_history_service_query_visits (EphyHistoryService     *self,
+                                   EphyHistoryQuery       *query,
+                                   GCancellable           *cancellable,
+                                   EphyHistoryJobCallback  callback,
+                                   gpointer                user_data)
 {
   EphyHistoryServiceMessage *message;
 
@@ -728,7 +759,9 @@ ephy_history_service_query_visits (EphyHistoryService *self, EphyHistoryQuery *q
 }
 
 static gboolean
-ephy_history_service_execute_query_urls (EphyHistoryService *self, EphyHistoryQuery *query, gpointer *result)
+ephy_history_service_execute_query_urls (EphyHistoryService *self,
+                                         EphyHistoryQuery   *query,
+                                         gpointer           *result)
 {
   GList *urls = ephy_history_service_find_url_rows (self, query);
 
@@ -738,7 +771,11 @@ ephy_history_service_execute_query_urls (EphyHistoryService *self, EphyHistoryQu
 }
 
 void
-ephy_history_service_query_urls (EphyHistoryService *self, EphyHistoryQuery *query, GCancellable *cancellable, EphyHistoryJobCallback callback, gpointer user_data)
+ephy_history_service_query_urls (EphyHistoryService     *self,
+                                 EphyHistoryQuery       *query,
+                                 GCancellable           *cancellable,
+                                 EphyHistoryJobCallback  callback,
+                                 gpointer                user_data)
 {
   EphyHistoryServiceMessage *message;
 
@@ -752,10 +789,10 @@ ephy_history_service_query_urls (EphyHistoryService *self, EphyHistoryQuery *que
 }
 
 void
-ephy_history_service_get_hosts (EphyHistoryService    *self,
-                                GCancellable          *cancellable,
-                                EphyHistoryJobCallback callback,
-                                gpointer               user_data)
+ephy_history_service_get_hosts (EphyHistoryService     *self,
+                                GCancellable           *cancellable,
+                                EphyHistoryJobCallback  callback,
+                                gpointer                user_data)
 {
   EphyHistoryServiceMessage *message;
 
@@ -768,11 +805,11 @@ ephy_history_service_get_hosts (EphyHistoryService    *self,
 }
 
 void
-ephy_history_service_query_hosts (EphyHistoryService    *self,
-                                  EphyHistoryQuery      *query,
-                                  GCancellable          *cancellable,
-                                  EphyHistoryJobCallback callback,
-                                  gpointer               user_data)
+ephy_history_service_query_hosts (EphyHistoryService     *self,
+                                  EphyHistoryQuery       *query,
+                                  GCancellable           *cancellable,
+                                  EphyHistoryJobCallback  callback,
+                                  gpointer                user_data)
 {
   EphyHistoryServiceMessage *message;
 
@@ -802,9 +839,6 @@ ephy_history_service_execute_set_url_title (EphyHistoryService *self,
 {
   char *title = g_strdup (url->title);
 
-  if (self->read_only)
-    return FALSE;
-
   if (ephy_history_service_get_url_row (self, NULL, url) == NULL) {
     /* The URL is not yet in the database, so we can't update it.. */
     g_free (title);
@@ -827,12 +861,12 @@ ephy_history_service_execute_set_url_title (EphyHistoryService *self,
 }
 
 void
-ephy_history_service_set_url_title (EphyHistoryService    *self,
-                                    const char            *orig_url,
-                                    const char            *title,
-                                    GCancellable          *cancellable,
-                                    EphyHistoryJobCallback callback,
-                                    gpointer               user_data)
+ephy_history_service_set_url_title (EphyHistoryService     *self,
+                                    const char             *orig_url,
+                                    const char             *title,
+                                    GCancellable           *cancellable,
+                                    EphyHistoryJobCallback  callback,
+                                    gpointer                user_data)
 {
   EphyHistoryURL *url;
   EphyHistoryServiceMessage *message;
@@ -858,9 +892,6 @@ ephy_history_service_execute_set_url_zoom_level (EphyHistoryService *self,
   double zoom_level;
   EphyHistoryHost *host;
 
-  if (self->read_only)
-    return FALSE;
-
   g_variant_get (variant, "(sd)", &url_string, &zoom_level);
 
   host = ephy_history_service_get_host_row_from_url (self, url_string);
@@ -875,12 +906,12 @@ ephy_history_service_execute_set_url_zoom_level (EphyHistoryService *self,
 }
 
 void
-ephy_history_service_set_url_zoom_level (EphyHistoryService    *self,
-                                         const char            *url,
-                                         const double           zoom_level,
-                                         GCancellable          *cancellable,
-                                         EphyHistoryJobCallback callback,
-                                         gpointer               user_data)
+ephy_history_service_set_url_zoom_level (EphyHistoryService     *self,
+                                         const char             *url,
+                                         double                  zoom_level,
+                                         GCancellable           *cancellable,
+                                         EphyHistoryJobCallback  callback,
+                                         gpointer                user_data)
 {
   EphyHistoryServiceMessage *message;
   GVariant *variant;
@@ -888,6 +919,10 @@ ephy_history_service_set_url_zoom_level (EphyHistoryService    *self,
   g_assert (EPHY_IS_HISTORY_SERVICE (self));
   g_assert (url != NULL);
 
+  /* Ensure that a change value which equals default zoom level is stored as 0.0 */
+  if (zoom_level == g_settings_get_double (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL))
+    zoom_level = 0.0f;
+
   variant = g_variant_new ("(sd)", url, zoom_level);
 
   message = ephy_history_service_message_new (self, SET_URL_ZOOM_LEVEL,
@@ -903,9 +938,6 @@ ephy_history_service_execute_set_url_hidden (EphyHistoryService *self,
 {
   gboolean hidden;
 
-  if (self->read_only)
-    return FALSE;
-
   hidden = url->hidden;
 
   if (ephy_history_service_get_url_row (self, NULL, url) == NULL) {
@@ -919,12 +951,12 @@ ephy_history_service_execute_set_url_hidden (EphyHistoryService *self,
 }
 
 void
-ephy_history_service_set_url_hidden (EphyHistoryService    *self,
-                                     const char            *orig_url,
-                                     gboolean               hidden,
-                                     GCancellable          *cancellable,
-                                     EphyHistoryJobCallback callback,
-                                     gpointer               user_data)
+ephy_history_service_set_url_hidden (EphyHistoryService     *self,
+                                     const char             *orig_url,
+                                     gboolean                hidden,
+                                     GCancellable           *cancellable,
+                                     EphyHistoryJobCallback  callback,
+                                     gpointer                user_data)
 {
   EphyHistoryServiceMessage *message;
   EphyHistoryURL *url;
@@ -956,11 +988,11 @@ ephy_history_service_execute_get_url (EphyHistoryService *self,
 }
 
 void
-ephy_history_service_get_url (EphyHistoryService    *self,
-                              const char            *url,
-                              GCancellable          *cancellable,
-                              EphyHistoryJobCallback callback,
-                              gpointer               user_data)
+ephy_history_service_get_url (EphyHistoryService     *self,
+                              const char             *url,
+                              GCancellable           *cancellable,
+                              EphyHistoryJobCallback  callback,
+                              gpointer                user_data)
 {
   EphyHistoryServiceMessage *message;
 
@@ -989,11 +1021,11 @@ ephy_history_service_execute_get_host_for_url (EphyHistoryService *self,
 }
 
 void
-ephy_history_service_get_host_for_url (EphyHistoryService    *self,
-                                       const char            *url,
-                                       GCancellable          *cancellable,
-                                       EphyHistoryJobCallback callback,
-                                       gpointer               user_data)
+ephy_history_service_get_host_for_url (EphyHistoryService     *self,
+                                       const char             *url,
+                                       GCancellable           *cancellable,
+                                       EphyHistoryJobCallback  callback,
+                                       gpointer                user_data)
 {
   EphyHistoryServiceMessage *message;
 
@@ -1025,9 +1057,6 @@ ephy_history_service_execute_delete_urls (EphyHistoryService *self,
   EphyHistoryURL *url;
   SignalEmissionContext *ctx;
 
-  if (self->read_only)
-    return FALSE;
-
   for (l = urls; l != NULL; l = l->next) {
     url = l->data;
     ephy_history_service_delete_url (self, url);
@@ -1058,16 +1087,13 @@ delete_host_signal_emit (SignalEmissionContext *ctx)
 }
 
 static gboolean
-ephy_history_service_execute_delete_host (EphyHistoryService    *self,
-                                          EphyHistoryHost       *host,
-                                          EphyHistoryJobCallback callback,
-                                          gpointer               user_data)
+ephy_history_service_execute_delete_host (EphyHistoryService     *self,
+                                          EphyHistoryHost        *host,
+                                          EphyHistoryJobCallback  callback,
+                                          gpointer                user_data)
 {
   SignalEmissionContext *ctx;
 
-  if (self->read_only)
-    return FALSE;
-
   ephy_history_service_delete_host_row (self, host);
 
   ctx = signal_emission_context_new (self, g_strdup (host->url),
@@ -1085,8 +1111,7 @@ ephy_history_service_execute_clear (EphyHistoryService *self,
                                     gpointer            pointer,
                                     gpointer           *result)
 {
-  if (self->history_database == NULL ||
-      self->read_only)
+  if (self->history_database == NULL)
     return FALSE;
 
   ephy_history_service_commit_transaction (self);
@@ -1100,11 +1125,11 @@ ephy_history_service_execute_clear (EphyHistoryService *self,
 }
 
 void
-ephy_history_service_delete_urls (EphyHistoryService    *self,
-                                  GList                 *urls,
-                                  GCancellable          *cancellable,
-                                  EphyHistoryJobCallback callback,
-                                  gpointer               user_data)
+ephy_history_service_delete_urls (EphyHistoryService     *self,
+                                  GList                  *urls,
+                                  GCancellable           *cancellable,
+                                  EphyHistoryJobCallback  callback,
+                                  gpointer                user_data)
 {
   EphyHistoryServiceMessage *message;
 
@@ -1118,11 +1143,11 @@ ephy_history_service_delete_urls (EphyHistoryService    *self,
 }
 
 void
-ephy_history_service_delete_host (EphyHistoryService    *self,
-                                  EphyHistoryHost       *host,
-                                  GCancellable          *cancellable,
-                                  EphyHistoryJobCallback callback,
-                                  gpointer               user_data)
+ephy_history_service_delete_host (EphyHistoryService     *self,
+                                  EphyHistoryHost        *host,
+                                  GCancellable           *cancellable,
+                                  EphyHistoryJobCallback  callback,
+                                  gpointer                user_data)
 {
   EphyHistoryServiceMessage *message =
     ephy_history_service_message_new (self, DELETE_HOST,
@@ -1132,10 +1157,10 @@ ephy_history_service_delete_host (EphyHistoryService    *self,
 }
 
 void
-ephy_history_service_clear (EphyHistoryService    *self,
-                            GCancellable          *cancellable,
-                            EphyHistoryJobCallback callback,
-                            gpointer               user_data)
+ephy_history_service_clear (EphyHistoryService     *self,
+                            GCancellable           *cancellable,
+                            EphyHistoryJobCallback  callback,
+                            gpointer                user_data)
 {
   EphyHistoryServiceMessage *message;
 
@@ -1148,9 +1173,9 @@ ephy_history_service_clear (EphyHistoryService    *self,
 }
 
 static void
-ephy_history_service_quit (EphyHistoryService    *self,
-                           EphyHistoryJobCallback callback,
-                           gpointer               user_data)
+ephy_history_service_quit (EphyHistoryService     *self,
+                           EphyHistoryJobCallback  callback,
+                           gpointer                user_data)
 {
   EphyHistoryServiceMessage *message =
     ephy_history_service_message_new (self, QUIT,
@@ -1218,14 +1243,16 @@ ephy_history_service_process_message (EphyHistoryService        *self,
 /* Public API. */
 
 void
-ephy_history_service_find_urls (EphyHistoryService *self,
-                                gint64 from, gint64 to,
-                                guint limit, gint host,
-                                GList *substring_list,
-                                EphyHistorySortType sort_type,
-                                GCancellable *cancellable,
-                                EphyHistoryJobCallback callback,
-                                gpointer user_data)
+ephy_history_service_find_urls (EphyHistoryService     *self,
+                                gint64                  from,
+                                gint64                  to,
+                                guint                   limit,
+                                gint                    host,
+                                GList                  *substring_list,
+                                EphyHistorySortType     sort_type,
+                                GCancellable           *cancellable,
+                                EphyHistoryJobCallback  callback,
+                                gpointer                user_data)
 {
   EphyHistoryQuery *query;
 
@@ -1271,11 +1298,12 @@ ephy_history_service_visit_url (EphyHistoryService       *self,
 }
 
 void
-ephy_history_service_find_hosts (EphyHistoryService *self,
-                                 gint64 from, gint64 to,
-                                 GCancellable *cancellable,
-                                 EphyHistoryJobCallback callback,
-                                 gpointer user_data)
+ephy_history_service_find_hosts (EphyHistoryService     *self,
+                                 gint64                  from,
+                                 gint64                  to,
+                                 GCancellable           *cancellable,
+                                 EphyHistoryJobCallback  callback,
+                                 gpointer                user_data)
 {
   EphyHistoryQuery *query;
 
