@@ -44,10 +44,10 @@
  * The format is {THREAT_TYPE, PLATFORM_TYPE, THREAT_ENTRY_TYPE}.
  */
 static const char * const gsb_linux_threat_lists[][3] = {
-  {GSB_THREAT_TYPE_MALWARE,            "LINUX",        "URL"},
+  {GSB_THREAT_TYPE_MALWARE, "LINUX", "URL"},
   {GSB_THREAT_TYPE_SOCIAL_ENGINEERING, "ANY_PLATFORM", "URL"},
-  {GSB_THREAT_TYPE_UNWANTED_SOFTWARE,  "LINUX",        "URL"},
-  {GSB_THREAT_TYPE_MALWARE,            "LINUX",        "IP_RANGE"},
+  {GSB_THREAT_TYPE_UNWANTED_SOFTWARE, "LINUX", "URL"},
+  {GSB_THREAT_TYPE_MALWARE, "LINUX", "IP_RANGE"},
 };
 
 struct _EphyGSBStorage {
@@ -124,7 +124,9 @@ ephy_gsb_storage_start_transaction (EphyGSBStorage *self)
   GError *error = NULL;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
+
+  if (!self->is_operable)
+    return;
 
   ephy_sqlite_connection_begin_transaction (self->db, &error);
   if (error) {
@@ -139,7 +141,9 @@ ephy_gsb_storage_end_transaction (EphyGSBStorage *self)
   GError *error = NULL;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
+
+  if (!self->is_operable)
+    return;
 
   ephy_sqlite_connection_commit_transaction (self->db, &error);
   if (error) {
@@ -382,11 +386,12 @@ static void
 ephy_gsb_storage_clear_db (EphyGSBStorage *self)
 {
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (EPHY_IS_SQLITE_CONNECTION (self->db));
 
-  ephy_sqlite_connection_close (self->db);
-  ephy_sqlite_connection_delete_database (self->db);
-  g_clear_object (&self->db);
+  if (self->db) {
+    ephy_sqlite_connection_close (self->db);
+    ephy_sqlite_connection_delete_database (self->db);
+    g_clear_object (&self->db);
+  }
 }
 
 static gboolean
@@ -408,6 +413,8 @@ ephy_gsb_storage_init_db (EphyGSBStorage *self)
   if (!success)
     ephy_gsb_storage_clear_db (self);
 
+  self->is_operable = success;
+
   return success;
 }
 
@@ -492,17 +499,19 @@ ephy_gsb_storage_constructed (GObject *object)
 
   if (!g_file_test (self->db_path, G_FILE_TEST_EXISTS)) {
     LOG ("GSB database does not exist, initializing...");
-    success = ephy_gsb_storage_init_db (self);
+    ephy_gsb_storage_init_db (self);
   } else {
     LOG ("GSB database exists, opening...");
     success = ephy_gsb_storage_open_db (self);
-    if (success && !ephy_gsb_storage_check_schema_version (self)) {
-      LOG ("GSB database schema incompatibility, recreating database...");
-      success = ephy_gsb_storage_recreate_db (self);
+    if (success) {
+      if (!ephy_gsb_storage_check_schema_version (self)) {
+        LOG ("GSB database schema incompatibility, recreating database...");
+        ephy_gsb_storage_recreate_db (self);
+      } else {
+        self->is_operable = TRUE;
+      }
     }
   }
-
-  self->is_operable = success;
 }
 
 static void
@@ -627,9 +636,11 @@ ephy_gsb_storage_set_metadata (EphyGSBStorage *self,
   const char *sql;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (key);
 
+  if (!self->is_operable)
+    return;
+
   sql = "UPDATE metadata SET value=? WHERE key=?";
   statement = ephy_sqlite_connection_create_statement (self->db, sql, &error);
   if (error) {
@@ -686,7 +697,9 @@ ephy_gsb_storage_get_threat_lists (EphyGSBStorage *self)
   const char *sql;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
+
+  if (!self->is_operable)
+    return NULL;
 
   sql = "SELECT threat_type, platform_type, threat_entry_type, client_state FROM threats";
   statement = ephy_sqlite_connection_create_statement (self->db, sql, &error);
@@ -737,14 +750,16 @@ ephy_gsb_storage_compute_checksum (EphyGSBStorage    *self,
   GError *error = NULL;
   const char *sql;
   char *retval = NULL;
-  GChecksum *checksum ;
+  GChecksum *checksum;
   guint8 *digest;
   gsize digest_len = GSB_HASH_SIZE;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (list);
 
+  if (!self->is_operable)
+    return NULL;
+
   sql = "SELECT value FROM hash_prefix WHERE "
         "threat_type=? AND platform_type=? AND threat_entry_type=? "
         "ORDER BY value";
@@ -807,9 +822,11 @@ ephy_gsb_storage_update_client_state (EphyGSBStorage    *self,
   gboolean success;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (list);
 
+  if (!self->is_operable)
+    return;
+
   if (clear) {
     sql = "UPDATE threats SET client_state=NULL "
           "WHERE threat_type=? AND platform_type=? AND threat_entry_type=?";
@@ -861,9 +878,11 @@ ephy_gsb_storage_clear_hash_prefixes (EphyGSBStorage    *self,
   const char *sql;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (list);
 
+  if (!self->is_operable)
+    return;
+
   sql = "DELETE FROM hash_prefix WHERE "
         "threat_type=? AND platform_type=? AND threat_entry_type=?";
   statement = ephy_sqlite_connection_create_statement (self->db, sql, &error);
@@ -901,10 +920,12 @@ ephy_gsb_storage_get_hash_prefixes_to_delete (EphyGSBStorage    *self,
   guint index = 0;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (list);
   g_assert (indices);
 
+  if (!self->is_operable)
+    return NULL;
+
   *num_prefixes = 0;
 
   sql = "SELECT value FROM hash_prefix WHERE "
@@ -952,7 +973,9 @@ ephy_gsb_storage_make_delete_hash_prefix_statement (EphyGSBStorage *self,
   GString *sql;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
+
+  if (!self->is_operable)
+    return NULL;
 
   sql = g_string_new ("DELETE FROM hash_prefix WHERE "
                       "threat_type=? AND platform_type=? and threat_entry_type=? "
@@ -985,10 +1008,12 @@ ephy_gsb_storage_delete_hash_prefixes_batch (EphyGSBStorage      *self,
   gboolean free_statement = TRUE;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (list);
   g_assert (prefixes);
 
+  if (!self->is_operable)
+    return NULL;
+
   if (stmt) {
     statement = stmt;
     ephy_sqlite_statement_reset (statement);
@@ -1042,10 +1067,12 @@ ephy_gsb_storage_delete_hash_prefixes_internal (EphyGSBStorage    *self,
   gsize num_prefixes;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (list);
   g_assert (indices);
 
+  if (!self->is_operable)
+    return;
+
   LOG ("Deleting %lu hash prefixes...", num_indices);
 
   /* Move indices from the array to a hash table set. */
@@ -1105,10 +1132,12 @@ ephy_gsb_storage_delete_hash_prefixes (EphyGSBStorage    *self,
   gsize num_indices;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (list);
   g_assert (tes);
 
+  if (!self->is_operable)
+    return;
+
   compression = json_object_get_string_member (tes, "compressionType");
   if (!g_strcmp0 (compression, GSB_COMPRESSION_TYPE_RICE)) {
     rice_indices = json_object_get_object_member (tes, "riceIndices");
@@ -1137,7 +1166,9 @@ ephy_gsb_storage_make_insert_hash_prefix_statement (EphyGSBStorage *self,
   GString *sql;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
+
+  if (!self->is_operable)
+    return NULL;
 
   sql = g_string_new ("INSERT INTO hash_prefix "
                       "(cue, value, threat_type, platform_type, threat_entry_type) VALUES ");
@@ -1172,10 +1203,12 @@ ephy_gsb_storage_insert_hash_prefixes_batch (EphyGSBStorage      *self,
   gboolean free_statement = TRUE;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (list);
   g_assert (prefixes);
 
+  if (!self->is_operable)
+    return;
+
   if (stmt) {
     statement = stmt;
     ephy_sqlite_statement_reset (statement);
@@ -1219,10 +1252,12 @@ ephy_gsb_storage_insert_hash_prefixes_internal (EphyGSBStorage    *self,
   gsize num_batches;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (list);
   g_assert (prefixes);
 
+  if (!self->is_operable)
+    return;
+
   LOG ("Inserting %lu hash prefixes of size %ld...", num_prefixes, prefix_len);
 
   ephy_gsb_storage_start_transaction (self);
@@ -1280,10 +1315,12 @@ ephy_gsb_storage_insert_hash_prefixes (EphyGSBStorage    *self,
   gsize num_prefixes;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (list);
   g_assert (tes);
 
+  if (!self->is_operable)
+    return;
+
   compression = json_object_get_string_member (tes, "compressionType");
   if (!g_strcmp0 (compression, GSB_COMPRESSION_TYPE_RICE)) {
     rice_hashes = json_object_get_object_member (tes, "riceHashes");
@@ -1335,9 +1372,11 @@ ephy_gsb_storage_lookup_hash_prefixes (EphyGSBStorage *self,
   guint id = 0;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (cues);
 
+  if (!self->is_operable)
+    return NULL;
+
   sql = g_string_new ("SELECT value, negative_expires_at <= (CAST(strftime('%s', 'now') AS INT)) "
                       "FROM hash_prefix WHERE cue IN (");
   for (GList *l = cues; l && l->data; l = l->next)
@@ -1412,9 +1451,11 @@ ephy_gsb_storage_lookup_full_hashes (EphyGSBStorage *self,
   guint id = 0;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (hashes);
 
+  if (!self->is_operable)
+    return NULL;
+
   sql = g_string_new ("SELECT value, threat_type, platform_type, threat_entry_type, "
                       "expires_at <= (CAST(strftime('%s', 'now') AS INT)) "
                       "FROM hash_full WHERE value IN (");
@@ -1494,10 +1535,12 @@ ephy_gsb_storage_insert_full_hash (EphyGSBStorage    *self,
   const char *sql;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (list);
   g_assert (hash);
 
+  if (!self->is_operable)
+    return;
+
   LOG ("Inserting full hash with duration %ld for list %s/%s/%s",
        duration, list->threat_type, list->platform_type, list->threat_entry_type);
 
@@ -1576,7 +1619,9 @@ ephy_gsb_storage_delete_old_full_hashes (EphyGSBStorage *self)
   const char *sql;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
+
+  if (!self->is_operable)
+    return;
 
   LOG ("Deleting full hashes expired for more than %d seconds", EXPIRATION_THRESHOLD);
 
@@ -1625,9 +1670,11 @@ ephy_gsb_storage_update_hash_prefix_expiration (EphyGSBStorage *self,
   const char *sql;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
-  g_assert (self->is_operable);
   g_assert (prefix);
 
+  if (!self->is_operable)
+    return;
+
   sql = "UPDATE hash_prefix "
         "SET negative_expires_at=(CAST(strftime('%s', 'now') AS INT)) + ? "
         "WHERE value=?";
