@@ -55,6 +55,10 @@ struct _EphyPasswordManager {
   GHashTable *cache;
 };
 
+static void ephy_password_manager_forget_record (EphyPasswordManager *self,
+                                                 EphyPasswordRecord  *record,
+                                                 EphyPasswordRecord  *replacement);
+
 static void ephy_synchronizable_manager_iface_init (EphySynchronizableManagerInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (EphyPasswordManager, ephy_password_manager, G_TYPE_OBJECT,
@@ -64,25 +68,27 @@ G_DEFINE_TYPE_WITH_CODE (EphyPasswordManager, ephy_password_manager, G_TYPE_OBJE
 typedef struct {
   EphyPasswordManagerQueryCallback callback;
   gpointer user_data;
+  GList *records;
+  guint n_matches;
 } QueryAsyncData;
 
 typedef struct {
   EphyPasswordManager *manager;
-  char                *password;
+  char *password;
 } UpdatePasswordAsyncData;
 
 typedef struct {
   EphyPasswordManager *manager;
-  EphyPasswordRecord  *record;
+  EphyPasswordRecord *record;
 } ManageRecordAsyncData;
 
 typedef struct {
-  EphyPasswordManager                    *manager;
-  gboolean                                is_initial;
-  GList                                  *remotes_deleted;
-  GList                                  *remotes_updated;
-  EphySynchronizableManagerMergeCallback  callback;
-  gpointer                                user_data;
+  EphyPasswordManager *manager;
+  gboolean is_initial;
+  GList *remotes_deleted;
+  GList *remotes_updated;
+  EphySynchronizableManagerMergeCallback callback;
+  gpointer user_data;
 } MergePasswordsAsyncData;
 
 static QueryAsyncData *
@@ -91,7 +97,7 @@ query_async_data_new (EphyPasswordManagerQueryCallback callback,
 {
   QueryAsyncData *data;
 
-  data = g_new (QueryAsyncData, 1);
+  data = g_new0 (QueryAsyncData, 1);
   data->callback = callback;
   data->user_data = user_data;
 
@@ -103,6 +109,7 @@ query_async_data_free (QueryAsyncData *data)
 {
   g_assert (data);
 
+  g_list_free_full (data->records, g_object_unref);
   g_free (data);
 }
 
@@ -112,7 +119,7 @@ update_password_async_data_new (EphyPasswordManager *manager,
 {
   UpdatePasswordAsyncData *data;
 
-  data = g_new (UpdatePasswordAsyncData, 1);
+  data = g_new0 (UpdatePasswordAsyncData, 1);
   data->manager = g_object_ref (manager);
   data->password = g_strdup (password);
 
@@ -139,7 +146,7 @@ merge_passwords_async_data_new (EphyPasswordManager                    *manager,
 {
   MergePasswordsAsyncData *data;
 
-  data = g_new (MergePasswordsAsyncData, 1);
+  data = g_new0 (MergePasswordsAsyncData, 1);
   data->manager = g_object_ref (manager);
   data->is_initial = is_initial;
   data->remotes_deleted = remotes_deleted;
@@ -156,6 +163,8 @@ merge_passwords_async_data_free (MergePasswordsAsyncData *data)
   g_assert (data);
 
   g_object_unref (data->manager);
+  g_list_free_full (data->remotes_deleted, g_object_unref);
+  g_list_free_full (data->remotes_updated, g_object_unref);
   g_free (data);
 }
 
@@ -165,7 +174,7 @@ manage_record_async_data_new (EphyPasswordManager *manager,
 {
   ManageRecordAsyncData *data;
 
-  data = g_new (ManageRecordAsyncData, 1);
+  data = g_new0 (ManageRecordAsyncData, 1);
   data->manager = g_object_ref (manager);
   data->record = g_object_ref (record);
 
@@ -220,7 +229,7 @@ get_attributes_table (const char *id,
   if (server_time_modified >= 0)
     g_hash_table_insert (attributes,
                          g_strdup (SERVER_TIME_MODIFIED_KEY),
-                         g_strdup_printf ("%"PRId64, server_time_modified));
+                         g_strdup_printf ("%" PRId64, server_time_modified));
 
   return attributes;
 }
@@ -300,8 +309,6 @@ populate_cache_cb (GList    *records,
 
     ephy_password_manager_cache_add (self, origin, username);
   }
-
-  g_list_free_full (records, g_object_unref);
 }
 
 static void
@@ -341,8 +348,8 @@ ephy_password_manager_new (void)
 }
 
 GList *
-ephy_password_manager_get_cached_users (EphyPasswordManager *self,
-                                        const char          *origin)
+ephy_password_manager_get_usernames_for_origin (EphyPasswordManager *self,
+                                                const char          *origin)
 {
   g_assert (EPHY_IS_PASSWORD_MANAGER (self));
   g_assert (origin);
@@ -351,9 +358,9 @@ ephy_password_manager_get_cached_users (EphyPasswordManager *self,
 }
 
 static void
-secret_service_store_cb (SecretService         *service,
-                         GAsyncResult          *result,
-                         ManageRecordAsyncData *data)
+secret_password_store_cb (GObject               *source_object,
+                          GAsyncResult          *result,
+                          ManageRecordAsyncData *data)
 {
   GError *error = NULL;
   const char *origin;
@@ -362,7 +369,7 @@ secret_service_store_cb (SecretService         *service,
   origin = ephy_password_record_get_origin (data->record);
   username = ephy_password_record_get_username (data->record);
 
-  secret_service_store_finish (service, result, &error);
+  secret_password_store_finish (result, &error);
   if (error) {
     g_warning ("Failed to store password record for (%s, %s, %s, %s, %s): %s",
                origin,
@@ -384,7 +391,6 @@ ephy_password_manager_store_record (EphyPasswordManager *self,
                                     EphyPasswordRecord  *record)
 {
   GHashTable *attributes;
-  SecretValue *value;
   const char *origin;
   const char *target_origin;
   const char *username;
@@ -424,17 +430,39 @@ ephy_password_manager_store_record (EphyPasswordManager *self,
                                      username_field, password_field,
                                      modified);
 
-  value = secret_value_new (password, -1, "text/plain");
-  secret_service_store (NULL, EPHY_FORM_PASSWORD_SCHEMA,
-                        attributes, NULL, label, value, NULL,
-                        (GAsyncReadyCallback)secret_service_store_cb,
-                        manage_record_async_data_new (self, record));
+  secret_password_storev (EPHY_FORM_PASSWORD_SCHEMA,
+                          attributes, NULL, label, password, NULL,
+                          (GAsyncReadyCallback)secret_password_store_cb,
+                          manage_record_async_data_new (self, record));
 
   g_free (label);
-  secret_value_unref (value);
   g_hash_table_unref (attributes);
 }
 
+static GList *
+deduplicate_records (EphyPasswordManager *manager,
+                     GList               *records)
+{
+  GList *newest = records;
+  guint64 newest_modified = ephy_password_record_get_time_password_changed (newest->data);
+
+  for (GList *l = records->next; l; l = l->next) {
+    guint64 modified = ephy_password_record_get_time_password_changed (l->data);
+    if (modified > newest_modified) {
+      newest = l;
+      newest_modified = modified;
+    }
+  }
+
+  records = g_list_remove_link (records, newest);
+
+  for (GList *l = records; l; l = l->next)
+    ephy_password_manager_forget_record (manager, l->data, NULL);
+  g_list_free_full (records, g_object_unref);
+
+  return newest;
+}
+
 static void
 update_password_cb (GList    *records,
                     gpointer  user_data)
@@ -442,17 +470,21 @@ update_password_cb (GList    *records,
   UpdatePasswordAsyncData *data = (UpdatePasswordAsyncData *)user_data;
   EphyPasswordRecord *record;
 
-  /* We expect only one matching record here. */
-  if (g_list_length (records) == 1) {
+  /* Since we didn't include ID in our query, there could be multiple records
+   * returned. We only want to have one saved at a time, so delete the rest.
+   */
+  if (g_list_length (records) > 1)
+    records = deduplicate_records (data->manager, records);
+
+  if (records) {
     record = EPHY_PASSWORD_RECORD (records->data);
     ephy_password_record_set_password (record, data->password);
     ephy_password_manager_store_record (data->manager, record);
     g_signal_emit_by_name (data->manager, "synchronizable-modified", record, FALSE);
   } else {
-    g_warn_if_reached ();
+    LOG ("Attempted to update password record that doesn't exist (likely Epiphany bug)");
   }
 
-  g_list_free_full (records, g_object_unref);
   update_password_async_data_free (data);
 }
 
@@ -511,63 +543,110 @@ ephy_password_manager_save (EphyPasswordManager *self,
 }
 
 static void
-secret_service_search_cb (SecretService  *service,
-                          GAsyncResult   *result,
-                          QueryAsyncData *data)
+retrieve_secret_cb (GObject        *source_object,
+                    GAsyncResult   *result,
+                    QueryAsyncData *data)
 {
-  GList *matches = NULL;
-  GList *records = NULL;
+  SecretRetrievable *retrievable = SECRET_RETRIEVABLE (source_object);
+  GHashTable *attributes = NULL;
+  const char *id;
+  const char *origin;
+  const char *target_origin;
+  const char *username;
+  const char *username_field;
+  const char *password_field;
+  const char *timestamp;
+  gint64 created;
+  gint64 modified;
+  const char *password;
+  gint64 server_time_modified;
+  EphyPasswordRecord *record;
+  SecretValue *value = NULL;
   GError *error = NULL;
 
-  matches = secret_service_search_finish (service, result, &error);
-  if (error) {
-    g_warning ("Failed to search secrets in password schema: %s", error->message);
+  value = secret_retrievable_retrieve_secret_finish (retrievable, result, &error);
+  if (!value) {
+    g_warning ("Failed to retrieve password: %s", error->message);
     g_error_free (error);
     goto out;
   }
 
-  for (GList *l = matches; l && l->data; l = l->next) {
-    SecretItem *item = (SecretItem *)l->data;
-    GHashTable *attributes = secret_item_get_attributes (item);
-    SecretValue *value = secret_item_get_secret (item);
-    const char *id = g_hash_table_lookup (attributes, ID_KEY);
-    const char *origin = g_hash_table_lookup (attributes, ORIGIN_KEY);
-    const char *target_origin = g_hash_table_lookup (attributes, TARGET_ORIGIN_KEY);
-    const char *username = g_hash_table_lookup (attributes, USERNAME_KEY);
-    const char *username_field = g_hash_table_lookup (attributes, USERNAME_FIELD_KEY);
-    const char *password_field = g_hash_table_lookup (attributes, PASSWORD_FIELD_KEY);
-    const char *timestamp = g_hash_table_lookup (attributes, SERVER_TIME_MODIFIED_KEY);
-    const char *password = secret_value_get (value, NULL);
-    gint64 server_time_modified;
-    EphyPasswordRecord *record;
+  attributes = secret_retrievable_get_attributes (retrievable);
+  id = g_hash_table_lookup (attributes, ID_KEY);
+  origin = g_hash_table_lookup (attributes, ORIGIN_KEY);
+  target_origin = g_hash_table_lookup (attributes, TARGET_ORIGIN_KEY);
+  username = g_hash_table_lookup (attributes, USERNAME_KEY);
+  username_field = g_hash_table_lookup (attributes, USERNAME_FIELD_KEY);
+  password_field = g_hash_table_lookup (attributes, PASSWORD_FIELD_KEY);
+  timestamp = g_hash_table_lookup (attributes, SERVER_TIME_MODIFIED_KEY);
+  created = secret_retrievable_get_created (retrievable);
+  modified = secret_retrievable_get_modified (retrievable);
 
-    LOG ("Found password record for (%s, %s, %s, %s, %s)",
-         origin, target_origin, username, username_field, password_field);
+  LOG ("Found password record for (%s, %s, %s, %s, %s)",
+       origin, target_origin, username, username_field, password_field);
 
-    if (!id || !origin || !target_origin || !timestamp) {
-      LOG ("Password record is corrupted, skipping it...");
-      goto next;
-    }
-
-    record = ephy_password_record_new (id, origin, target_origin,
-                                       username, password,
-                                       username_field, password_field,
-                                       secret_item_get_created (item) * 1000,
-                                       secret_item_get_modified (item) * 1000);
-    server_time_modified = g_ascii_strtod (timestamp, NULL);
-    ephy_synchronizable_set_server_time_modified (EPHY_SYNCHRONIZABLE (record),
-                                                  server_time_modified);
-    records = g_list_prepend (records, record);
-
-next:
-    secret_value_unref (value);
-    g_hash_table_unref (attributes);
+  if (!id || !origin || !target_origin || !timestamp) {
+    LOG ("Password record is corrupted, skipping it...");
+    goto out;
   }
 
+  password = secret_value_get_text (value);
+
+  record = ephy_password_record_new (id, origin, target_origin,
+                                     username, password,
+                                     username_field, password_field,
+                                     created * 1000,
+                                     modified * 1000);
+  server_time_modified = g_ascii_strtod (timestamp, NULL);
+  ephy_synchronizable_set_server_time_modified (EPHY_SYNCHRONIZABLE (record),
+                                                server_time_modified);
+  data->records = g_list_prepend (data->records, record);
+
 out:
-  if (data->callback)
-    data->callback (records, data->user_data);
-  query_async_data_free (data);
+  if (value)
+    secret_value_unref (value);
+  if (attributes)
+    g_hash_table_unref (attributes);
+  g_object_unref (retrievable);
+
+  if (--data->n_matches == 0) {
+    if (data->callback)
+      data->callback (data->records, data->user_data);
+    query_async_data_free (data);
+  }
+}
+
+static void
+secret_password_search_cb (GObject        *source_object,
+                           GAsyncResult   *result,
+                           QueryAsyncData *data)
+{
+  GList *matches;
+  GError *error = NULL;
+
+  matches = secret_password_search_finish (result, &error);
+  if (!matches) {
+    if (error) {
+      g_warning ("Failed to search secrets in password schema: %s", error->message);
+      g_error_free (error);
+    }
+    if (data->callback)
+      data->callback (NULL, data->user_data);
+    query_async_data_free (data);
+    return;
+  }
+
+  data->records = NULL;
+  data->n_matches = g_list_length (matches);
+
+  for (GList *l = matches; l; l = l->next) {
+    SecretRetrievable *retrievable = SECRET_RETRIEVABLE (l->data);
+    secret_retrievable_retrieve_secret (g_object_ref (retrievable),
+                                        NULL,
+                                        (GAsyncReadyCallback)retrieve_secret_cb,
+                                        data);
+  }
+
   g_list_free_full (matches, g_object_unref);
 }
 
@@ -594,25 +673,24 @@ ephy_password_manager_query (EphyPasswordManager              *self,
                                      username_field, password_field, -1);
   data = query_async_data_new (callback, user_data);
 
-  secret_service_search (NULL,
-                         EPHY_FORM_PASSWORD_SCHEMA,
-                         attributes,
-                         SECRET_SEARCH_ALL | SECRET_SEARCH_UNLOCK | SECRET_SEARCH_LOAD_SECRETS,
-                         NULL,
-                         (GAsyncReadyCallback)secret_service_search_cb,
-                         data);
+  secret_password_searchv (EPHY_FORM_PASSWORD_SCHEMA,
+                           attributes,
+                           SECRET_SEARCH_ALL | SECRET_SEARCH_UNLOCK | SECRET_SEARCH_LOAD_SECRETS,
+                           NULL,
+                           (GAsyncReadyCallback)secret_password_search_cb,
+                           data);
 
   g_hash_table_unref (attributes);
 }
 
 static void
-secret_service_clear_cb (SecretService *service,
-                         GAsyncResult  *result,
-                         gpointer       user_data)
+secret_password_clear_cb (GObject      *source_object,
+                          GAsyncResult *result,
+                          gpointer      user_data)
 {
   GError *error = NULL;
 
-  secret_service_clear_finish (service, result, &error);
+  secret_password_clear_finish (result, &error);
   if (error) {
     g_warning ("Failed to clear secrets from password schema: %s", error->message);
     g_error_free (error);
@@ -651,9 +729,9 @@ ephy_password_manager_forget_record (EphyPasswordManager *self,
        ephy_password_record_get_username_field (record),
        ephy_password_record_get_password_field (record));
 
-  secret_service_clear (NULL, EPHY_FORM_PASSWORD_SCHEMA, attributes, NULL,
-                        (GAsyncReadyCallback)secret_service_clear_cb,
-                        replacement ? manage_record_async_data_new (self, replacement) : NULL);
+  secret_password_clearv (EPHY_FORM_PASSWORD_SCHEMA, attributes, NULL,
+                          (GAsyncReadyCallback)secret_password_clear_cb,
+                          replacement ? manage_record_async_data_new (self, replacement) : NULL);
 
   ephy_password_manager_cache_remove (self,
                                       ephy_password_record_get_origin (record),
@@ -676,8 +754,6 @@ forget_cb (GList    *records,
   } else {
     g_warn_if_reached ();
   }
-
-  g_list_free_full (records, g_object_unref);
 }
 
 void
@@ -688,8 +764,8 @@ ephy_password_manager_forget (EphyPasswordManager *self,
   g_assert (id);
 
   /* synchronizable-deleted signal needs an EphySynchronizable object,
-   * therefore we need to obtain the password record first and then emit
-   * the signal before clearing the password from the secret schema. */
+  * therefore we need to obtain the password record first and then emit
+  * the signal before clearing the password from the secret schema. */
   ephy_password_manager_query (self, id,
                                NULL, NULL, NULL, NULL, NULL,
                                forget_cb, self);
@@ -703,8 +779,8 @@ forget_all_cb (GList    *records,
   GHashTable *attributes;
 
   attributes = secret_attributes_build (EPHY_FORM_PASSWORD_SCHEMA, NULL);
-  secret_service_clear (NULL, EPHY_FORM_PASSWORD_SCHEMA, attributes, NULL,
-                        (GAsyncReadyCallback)secret_service_clear_cb, NULL);
+  secret_password_clearv (EPHY_FORM_PASSWORD_SCHEMA, attributes, NULL,
+                          (GAsyncReadyCallback)secret_password_clear_cb, NULL);
 
   for (GList *l = records; l && l->data; l = l->next)
     g_signal_emit_by_name (self, "synchronizable-deleted", l->data);
@@ -712,7 +788,6 @@ forget_all_cb (GList    *records,
   ephy_password_manager_cache_clear (self);
 
   g_hash_table_unref (attributes);
-  g_list_free_full (records, g_object_unref);
 }
 
 void
@@ -797,7 +872,6 @@ replace_existing_cb (GList    *records,
   else
     g_warn_if_reached ();
 
-  g_list_free_full (records, g_object_unref);
   manage_record_async_data_free (data);
 }
 
@@ -926,7 +1000,7 @@ ephy_password_manager_handle_initial_merge (EphyPasswordManager *self,
         local_timestamp = ephy_password_record_get_time_password_changed (record);
         if (local_timestamp > remote_timestamp) {
           /* Local record is newer. Keep it and upload it to server.
-           * Also, must keep the most recent server time modified. */
+          * Also, must keep the most recent server time modified. */
           local_server_time_modified = ephy_synchronizable_get_server_time_modified (EPHY_SYNCHRONIZABLE (record));
           if (local_server_time_modified < remote_server_time_modified) {
             ephy_synchronizable_set_server_time_modified (EPHY_SYNCHRONIZABLE (record),
@@ -1079,7 +1153,6 @@ merge_cb (GList    *records,
 
   data->callback (to_upload, data->user_data);
 
-  g_list_free_full (records, g_object_unref);
   merge_passwords_async_data_free (data);
 }
 
@@ -1097,8 +1170,8 @@ synchronizable_manager_merge (EphySynchronizableManager              *manager,
                                merge_cb,
                                merge_passwords_async_data_new (self,
                                                                is_initial,
-                                                               remotes_deleted,
-                                                               remotes_updated,
+                                                               g_list_copy_deep (remotes_deleted, (GCopyFunc)g_object_ref, NULL),
+                                                               g_list_copy_deep (remotes_updated, (GCopyFunc)g_object_ref, NULL),
                                                                callback,
                                                                user_data));
 }
