@@ -41,6 +41,8 @@ struct _EphySyncService {
   SoupSession *session;
   guint source_id;
 
+  GCancellable *cancellable;
+
   char *user;
   char *crypto_keys;
   GHashTable *secrets;
@@ -681,15 +683,16 @@ out:
 }
 
 static void
-forget_secrets_cb (SecretService *service,
-                   GAsyncResult  *result,
-                   gpointer       user_data)
+forget_secrets_cb (GObject      *source_object,
+                   GAsyncResult *result,
+                   gpointer      user_data)
 {
   GError *error = NULL;
 
-  secret_service_clear_finish (service, result, &error);
+  secret_password_clear_finish (result, &error);
   if (error) {
-    g_warning ("Failed to clear sync secrets: %s", error->message);
+    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      g_warning ("Failed to clear sync secrets: %s", error->message);
     g_error_free (error);
   } else {
     LOG ("Successfully cleared sync secrets");
@@ -711,8 +714,8 @@ ephy_sync_service_forget_secrets (EphySyncService *self)
   attributes = secret_attributes_build (EPHY_SYNC_SECRET_SCHEMA,
                                         EPHY_SYNC_SECRET_ACCOUNT_KEY, user,
                                         NULL);
-  secret_service_clear (NULL, EPHY_SYNC_SECRET_SCHEMA, attributes, NULL,
-                        (GAsyncReadyCallback)forget_secrets_cb, NULL);
+  secret_password_clearv (EPHY_SYNC_SECRET_SCHEMA, attributes, self->cancellable,
+                          (GAsyncReadyCallback)forget_secrets_cb, NULL);
   g_hash_table_remove_all (self->secrets);
 
   g_hash_table_unref (attributes);
@@ -1685,7 +1688,7 @@ sync_frequency_changed_cb (GSettings       *settings,
 }
 
 static void
-load_secrets_cb (SecretService   *service,
+load_secrets_cb (GObject         *source_object,
                  GAsyncResult    *result,
                  EphySyncService *self)
 {
@@ -1697,8 +1700,12 @@ load_secrets_cb (SecretService   *service,
   const char *message;
   const char *suggestion;
 
-  res = secret_service_search_finish (service, result, &error);
+  res = secret_password_search_finish (result, &error);
   if (error) {
+    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+      g_error_free (error);
+      goto out_no_error;
+    }
     g_warning ("Failed to search for sync secrets: %s", error->message);
     g_error_free (error);
     message = _("Could not find the sync secrets for the current sync user.");
@@ -1766,23 +1773,25 @@ ephy_sync_service_load_secrets (EphySyncService *self)
   attributes = secret_attributes_build (EPHY_SYNC_SECRET_SCHEMA,
                                         EPHY_SYNC_SECRET_ACCOUNT_KEY, user,
                                         NULL);
-  secret_service_search (NULL, EPHY_SYNC_SECRET_SCHEMA, attributes,
-                         SECRET_SEARCH_UNLOCK | SECRET_SEARCH_LOAD_SECRETS,
-                         NULL, (GAsyncReadyCallback)load_secrets_cb, self);
+  secret_password_searchv (EPHY_SYNC_SECRET_SCHEMA, attributes,
+                           SECRET_SEARCH_UNLOCK | SECRET_SEARCH_LOAD_SECRETS,
+                           self->cancellable, (GAsyncReadyCallback)load_secrets_cb, self);
 
   g_hash_table_unref (attributes);
   g_free (user);
 }
 
 static void
-store_secrets_cb (SecretService   *service,
+store_secrets_cb (GObject         *source_object,
                   GAsyncResult    *result,
                   EphySyncService *self)
 {
   GError *error = NULL;
 
-  secret_service_store_finish (service, result, &error);
+  secret_password_store_finish (result, &error);
   if (error) {
+    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      return;
     g_warning ("Failed to store sync secrets: %s", error->message);
     ephy_sync_service_destroy_session (self, NULL);
     g_hash_table_remove_all (self->secrets);
@@ -1804,7 +1813,6 @@ ephy_sync_service_store_secrets (EphySyncService *self)
 {
   JsonNode *node;
   JsonObject *object;
-  SecretValue *secret;
   GHashTable *attributes;
   GHashTableIter iter;
   gpointer key;
@@ -1824,7 +1832,6 @@ ephy_sync_service_store_secrets (EphySyncService *self)
   json_node_set_object (node, object);
   json_string = json_to_string (node, FALSE);
 
-  secret = secret_value_new (json_string, -1, "text/plain");
   attributes = secret_attributes_build (EPHY_SYNC_SECRET_SCHEMA,
                                         EPHY_SYNC_SECRET_ACCOUNT_KEY, self->user,
                                         NULL);
@@ -1832,13 +1839,12 @@ ephy_sync_service_store_secrets (EphySyncService *self)
   label = g_strdup_printf (_("The sync secrets of %s"), self->user);
 
   LOG ("Storing sync secrets...");
-  secret_service_store (NULL, EPHY_SYNC_SECRET_SCHEMA,
-                        attributes, NULL, label, secret, NULL,
-                        (GAsyncReadyCallback)store_secrets_cb, self);
+  secret_password_storev (EPHY_SYNC_SECRET_SCHEMA,
+                          attributes, NULL, label, json_string, NULL,
+                          (GAsyncReadyCallback)store_secrets_cb, self);
 
   g_free (label);
   g_free (json_string);
-  secret_value_unref (secret);
   g_hash_table_unref (attributes);
   json_object_unref (object);
   json_node_unref (node);
@@ -1947,6 +1953,11 @@ ephy_sync_service_dispose (GObject *object)
   g_clear_object (&self->session);
   g_clear_pointer (&self->secrets, g_hash_table_unref);
 
+  if (self->cancellable) {
+    g_cancellable_cancel (self->cancellable);
+    g_clear_object (&self->cancellable);
+  }
+
   G_OBJECT_CLASS (ephy_sync_service_parent_class)->dispose (object);
 }
 
@@ -1973,6 +1984,7 @@ ephy_sync_service_init (EphySyncService *self)
   self->session = soup_session_new ();
   self->storage_queue = g_queue_new ();
   self->secrets = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+  self->cancellable = g_cancellable_new ();
 
   if (ephy_sync_utils_user_is_signed_in ())
     ephy_sync_service_load_secrets (self);
