@@ -112,7 +112,7 @@ update_preview_cb (GtkFileChooser *file_chooser,
 
   if (!filename || g_stat (filename, &st_buf) || (!S_ISREG (st_buf.st_mode))) {
     gtk_file_chooser_set_preview_widget_active (file_chooser, FALSE);
-    return; // stat failed or file is not regular
+    return; /* stat failed or file is not regular */
   }
 
   if (!preview_format ||
@@ -120,7 +120,7 @@ update_preview_cb (GtkFileChooser *file_chooser,
       preview_width > MAX_PREVIEW_SOURCE_SIZE ||
       preview_height > MAX_PREVIEW_SOURCE_SIZE) {
     gtk_file_chooser_set_preview_widget_active (file_chooser, FALSE);
-    return; // unpreviewable, 0px, or unsafely large
+    return; /* unpreviewable, 0px, or unsafely large */
   }
 
   if (preview_width > MAX_PREVIEW_SIZE || preview_height > MAX_PREVIEW_SIZE) {
@@ -143,14 +143,14 @@ update_preview_cb (GtkFileChooser *file_chooser,
 }
 
 GtkFileChooser *
-ephy_create_file_chooser (const char           *title,
-                          GtkWidget            *parent,
-                          GtkFileChooserAction  action,
-                          EphyFileFilterDefault default_filter)
+ephy_create_file_chooser (const char            *title,
+                          GtkWidget             *parent,
+                          GtkFileChooserAction   action,
+                          EphyFileFilterDefault  default_filter)
 {
   GtkFileChooser *dialog;
   GtkFileFilter *filter[EPHY_FILE_FILTER_LAST];
-  char *downloads_dir;
+  g_autofree char *downloads_dir = NULL;
   GtkWidget *preview = gtk_image_new ();
 
   g_assert (GTK_IS_WINDOW (parent));
@@ -165,7 +165,6 @@ ephy_create_file_chooser (const char           *title,
 
   downloads_dir = ephy_file_get_downloads_dir ();
   gtk_file_chooser_add_shortcut_folder (dialog, downloads_dir, NULL);
-  g_free (downloads_dir);
 
   if (action == GTK_FILE_CHOOSER_ACTION_OPEN ||
       action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER ||
@@ -192,6 +191,7 @@ ephy_create_file_chooser (const char           *title,
         "message/rfc822",                                     /* MHTML */
         "multipart/related",                                  /* MHTML */
         "application/x-mimearchive",                          /* MHTML */
+        "application/pdf",
         "image/png",
         "image/jpeg",
         "image/gif",
