@@ -27,7 +27,7 @@
 #include <dazzle.h>
 #include <glib/gi18n.h>
 
-#define MAX_COMPLETION_HISTORY_URLS 8
+#define MAX_URL_ENTRIES             25
 
 struct _EphySuggestionModel {
   GObject parent;
@@ -36,6 +36,7 @@ struct _EphySuggestionModel {
   GSequence *urls;
   GSequence *items;
   GCancellable *icon_cancellable;
+  guint num_custom_entries;
 };
 
 enum {
@@ -193,13 +194,44 @@ static gboolean
 should_add_bookmark_to_model (EphySuggestionModel *self,
                               const char          *search_string,
                               const char          *title,
-                              const char          *location)
+                              const char          *location,
+                              GSequence           *tags)
 {
   g_autofree gchar *search_casefold = g_utf8_casefold (search_string, -1);
   g_autofree gchar *title_casefold = g_utf8_casefold (title, -1);
   g_autofree gchar *location_casefold = g_utf8_casefold (location, -1);
+  g_autofree char *tag_string = NULL;
+  g_autofree char *tag_string_casefold = NULL;
+  char **tag_array = NULL;
+  g_auto (GStrv) search_terms = NULL;
+  GSequenceIter *tag_iter;
+  guint i;
+  gboolean ret = TRUE;
 
-  return strstr (title_casefold, search_casefold) || strstr (location_casefold, search_casefold);
+  tag_array = g_malloc0 ((g_sequence_get_length (tags) + 1) * sizeof (char *));
+
+  for (i = 0, tag_iter = g_sequence_get_begin_iter (tags);
+       !g_sequence_iter_is_end (tag_iter);
+       i++, tag_iter = g_sequence_iter_next (tag_iter)) {
+    tag_array[i] = g_sequence_get (tag_iter);
+  }
+
+  tag_string = g_strjoinv (" ", tag_array);
+  tag_string_casefold = g_utf8_casefold (tag_string, -1);
+  search_terms = g_strsplit (search_casefold, " ", -1);
+
+  for (i = 0; i < g_strv_length (search_terms); i++) {
+    if (!strstr (title_casefold, search_terms[i]) &&
+        !strstr (location_casefold, search_terms[i]) &&
+        (tag_string_casefold && !strstr (tag_string_casefold, search_terms[i]))) {
+      ret = FALSE;
+      break;
+    }
+  }
+
+  g_free (tag_array);
+
+  return ret;
 }
 
 static void
@@ -247,6 +279,20 @@ load_favicon (EphySuggestionModel *model,
                                        suggestion);
 }
 
+static gboolean
+append_suggestion (EphySuggestionModel *self,
+                   EphySuggestion      *suggestion)
+{
+  if (self->num_custom_entries < MAX_URL_ENTRIES) {
+    g_sequence_append (self->items, suggestion);
+    self->num_custom_entries++;
+
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 static guint
 add_bookmarks (EphySuggestionModel *self,
                const char          *query)
@@ -261,21 +307,21 @@ add_bookmarks (EphySuggestionModel *self,
        !g_sequence_iter_is_end (iter);
        iter = g_sequence_iter_next (iter)) {
     EphyBookmark *bookmark;
+    GSequence *tags;
     const char *url, *title;
 
     bookmark = g_sequence_get (iter);
 
     url = ephy_bookmark_get_url (bookmark);
-    title = ephy_bookmark_get_title (bookmark);
-
-    if (g_sequence_lookup (self->urls, (gpointer)url, (GCompareDataFunc)g_strcmp0,
-                           NULL))
+    if (g_sequence_lookup (self->urls, (gpointer)url, (GCompareDataFunc)g_strcmp0, NULL))
       continue;
 
+    title = ephy_bookmark_get_title (bookmark);
     if (strlen (title) == 0)
       title = url;
 
-    if (should_add_bookmark_to_model (self, query, title, url)) {
+    tags = ephy_bookmark_get_tags (bookmark);
+    if (should_add_bookmark_to_model (self, query, title, url, tags)) {
       EphySuggestion *suggestion;
       g_autofree gchar *escaped_title = NULL;
       g_autofree gchar *markup = NULL;
@@ -286,9 +332,12 @@ add_bookmarks (EphySuggestionModel *self,
       load_favicon (self, suggestion, url);
       ephy_suggestion_set_secondary_icon (suggestion, "starred-symbolic");
 
-      new_urls = g_list_prepend (new_urls, g_strdup (url));
-      g_sequence_append (self->items, suggestion);
-      added++;
+      if (append_suggestion (self, suggestion)) {
+        new_urls = g_list_prepend (new_urls, g_strdup (url));
+        added++;
+      } else {
+        break;
+      }
     }
   }
 
@@ -326,8 +375,10 @@ add_history (EphySuggestionModel *self,
     suggestion = ephy_suggestion_new (markup, title, url->url);
     load_favicon (self, suggestion, url->url);
 
-    g_sequence_append (self->items, suggestion);
-    added++;
+    if (append_suggestion (self, suggestion))
+      added++;
+    else
+      break;
   }
 
   return added;
@@ -449,6 +500,32 @@ add_tabs (EphySuggestionModel *self,
   return added;
 }
 
+typedef struct {
+  char *query;
+  gboolean include_search_engines;
+} QueryData;
+
+static QueryData *
+query_data_new (const char *query,
+                gboolean    include_search_engines)
+{
+  QueryData *data;
+
+  data = g_malloc0 (sizeof (QueryData));
+  data->query = g_strdup (query);
+  data->include_search_engines = include_search_engines;
+
+  return data;
+}
+
+static void
+query_data_free (QueryData *data)
+{
+  g_assert (data != NULL);
+  g_free (data->query);
+  g_free (data);
+}
+
 static void
 query_completed_cb (EphyHistoryService *service,
                     gboolean            success,
@@ -457,13 +534,13 @@ query_completed_cb (EphyHistoryService *service,
 {
   GTask *task = user_data;
   EphySuggestionModel *self;
-  const gchar *query;
+  QueryData *data;
   GList *urls;
   guint removed;
   guint added = 0;
 
   self = g_task_get_source_object (task);
-  query = g_task_get_task_data (task);
+  data = g_task_get_task_data (task);
   urls = (GList *)result_data;
 
   g_cancellable_cancel (self->icon_cancellable);
@@ -477,12 +554,15 @@ query_completed_cb (EphyHistoryService *service,
   self->urls = g_sequence_new (g_free);
   g_clear_pointer (&self->items, g_sequence_free);
   self->items = g_sequence_new (g_object_unref);
+  self->num_custom_entries = 0;
 
-  if (strlen (query) > 0) {
-    added = add_tabs (self, query);
-    added += add_bookmarks (self, query);
-    added += add_history (self, urls, query);
-    added += add_search_engines (self, query);
+  if (strlen (data->query) > 0) {
+    added = add_tabs (self, data->query);
+    added += add_bookmarks (self, data->query);
+    added += add_history (self, urls, data->query);
+
+    if (data->include_search_engines)
+      added += add_search_engines (self, data->query);
   }
 
   g_list_model_items_changed (G_LIST_MODEL (self), 0, removed, added);
@@ -494,6 +574,7 @@ query_completed_cb (EphyHistoryService *service,
 void
 ephy_suggestion_model_query_async (EphySuggestionModel *self,
                                    const gchar         *query,
+                                   gboolean             include_search_engines,
                                    GCancellable        *cancellable,
                                    GAsyncReadyCallback  callback,
                                    gpointer             user_data)
@@ -501,6 +582,7 @@ ephy_suggestion_model_query_async (EphySuggestionModel *self,
   GTask *task = NULL;
   char **strings;
   GList *qlist = NULL;
+  QueryData *data;
 
   g_assert (EPHY_IS_SUGGESTION_MODEL (self));
   g_assert (query != NULL);
@@ -508,7 +590,9 @@ ephy_suggestion_model_query_async (EphySuggestionModel *self,
 
   task = g_task_new (self, cancellable, callback, user_data);
   g_task_set_source_tag (task, ephy_suggestion_model_query_async);
-  g_task_set_task_data (task, g_strdup (query), g_free);
+
+  data = query_data_new (query, include_search_engines);
+  g_task_set_task_data (task, data, (GDestroyNotify)query_data_free);
 
   /* Split the search string. */
   strings = g_strsplit (query, " ", -1);
@@ -517,7 +601,7 @@ ephy_suggestion_model_query_async (EphySuggestionModel *self,
 
   ephy_history_service_find_urls (self->history_service,
                                   0, 0,
-                                  MAX_COMPLETION_HISTORY_URLS, 0,
+                                  MAX_URL_ENTRIES, 0,
                                   qlist,
                                   EPHY_HISTORY_SORT_MOST_VISITED,
                                   cancellable,
