@@ -39,7 +39,7 @@ typedef enum {
 } PasswordsDialogColumn;
 
 struct _EphyPasswordsDialog {
-  GtkDialog parent_instance;
+  EphyDataDialog parent_instance;
 
   EphyPasswordManager *manager;
   GList *records;
@@ -54,13 +54,9 @@ struct _EphyPasswordsDialog {
   GMenuModel *treeview_popup_menu_model;
 
   GActionGroup *action_group;
-
-  gboolean filled;
-
-  char *search_text;
 };
 
-G_DEFINE_TYPE (EphyPasswordsDialog, ephy_passwords_dialog, GTK_TYPE_DIALOG)
+G_DEFINE_TYPE (EphyPasswordsDialog, ephy_passwords_dialog, EPHY_TYPE_DATA_DIALOG)
 
 enum {
   PROP_0,
@@ -114,9 +110,6 @@ ephy_passwords_dialog_dispose (GObject *object)
 
   g_clear_object (&dialog->manager);
 
-  g_free (dialog->search_text);
-  dialog->search_text = NULL;
-
   g_list_free_full (dialog->records, g_object_unref);
   dialog->records = NULL;
 
@@ -248,20 +241,14 @@ on_treeview_selection_changed (GtkTreeSelection    *selection,
 }
 
 static void
-on_search_entry_changed (GtkSearchEntry      *entry,
-                         EphyPasswordsDialog *dialog)
+on_search_text_changed (EphyPasswordsDialog *dialog)
 {
-  const char *text;
-
-  text = gtk_entry_get_text (GTK_ENTRY (entry));
-  g_free (dialog->search_text);
-  dialog->search_text = g_strdup (text);
   gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (dialog->treemodelfilter));
 }
 
 static char *
-get_selected_item (EphyPasswordsDialog  *dialog,
-                   PasswordsDialogColumn column)
+get_selected_item (EphyPasswordsDialog   *dialog,
+                   PasswordsDialogColumn  column)
 {
   GtkTreeModel *model;
   GList *selected;
@@ -384,7 +371,7 @@ ephy_passwords_dialog_class_init (EphyPasswordsDialogClass *klass)
 
   gtk_widget_class_bind_template_callback (widget_class, on_passwords_treeview_button_press_event);
   gtk_widget_class_bind_template_callback (widget_class, on_treeview_selection_changed);
-  gtk_widget_class_bind_template_callback (widget_class, on_search_entry_changed);
+  gtk_widget_class_bind_template_callback (widget_class, on_search_text_changed);
 }
 
 static void
@@ -397,7 +384,7 @@ forget_all (GSimpleAction *action,
   ephy_password_manager_forget_all (dialog->manager);
 
   gtk_list_store_clear (GTK_LIST_STORE (dialog->liststore));
-  dialog->filled = FALSE;
+  ephy_data_dialog_set_has_data (EPHY_DATA_DIALOG (dialog), FALSE);
 
   g_list_free_full (dialog->records, g_object_unref);
   dialog->records = NULL;
@@ -409,6 +396,7 @@ populate_model_cb (GList    *records,
 {
   EphyPasswordsDialog *dialog = EPHY_PASSWORDS_DIALOG (user_data);
 
+  ephy_data_dialog_set_is_loading (EPHY_DATA_DIALOG (dialog), FALSE);
   for (GList *l = records; l && l->data; l = l->next) {
     EphyPasswordRecord *record = EPHY_PASSWORD_RECORD (l->data);
     GtkTreeIter iter;
@@ -422,17 +410,20 @@ populate_model_cb (GList    *records,
                                        COL_PASSWORDS_INVISIBLE, "●●●●●●●●",
                                        COL_PASSWORDS_DATA, record,
                                        -1);
+    ephy_data_dialog_set_has_data (EPHY_DATA_DIALOG (dialog), TRUE);
   }
 
-  dialog->records = records;
+  g_assert (!dialog->records);
+  dialog->records = g_list_copy_deep (records, (GCopyFunc)g_object_ref, NULL);
 }
 
 static void
 populate_model (EphyPasswordsDialog *dialog)
 {
   g_assert (EPHY_IS_PASSWORDS_DIALOG (dialog));
-  g_assert (dialog->filled == FALSE);
+  g_assert (!ephy_data_dialog_get_has_data (EPHY_DATA_DIALOG (dialog)));
 
+  ephy_data_dialog_set_is_loading (EPHY_DATA_DIALOG (dialog), TRUE);
   /* Ask for all password records. */
   ephy_password_manager_query (dialog->manager,
                                NULL, NULL, NULL, NULL, NULL, NULL,
@@ -447,8 +438,9 @@ row_visible_func (GtkTreeModel        *model,
   char *username;
   char *origin;
   gboolean visible = FALSE;
+  const char *search_text = ephy_data_dialog_get_search_text (EPHY_DATA_DIALOG (dialog));
 
-  if (dialog->search_text == NULL)
+  if (search_text == NULL)
     return TRUE;
 
   gtk_tree_model_get (model, iter,
@@ -456,9 +448,9 @@ row_visible_func (GtkTreeModel        *model,
                       COL_PASSWORDS_USER, &username,
                       -1);
 
-  if (origin != NULL && g_strrstr (origin, dialog->search_text) != NULL)
+  if (origin != NULL && g_strrstr (origin, search_text) != NULL)
     visible = TRUE;
-  else if (username != NULL && g_strrstr (username, dialog->search_text) != NULL)
+  else if (username != NULL && g_strrstr (username, search_text) != NULL)
     visible = TRUE;
 
   g_free (origin);
@@ -518,6 +510,5 @@ ephy_passwords_dialog_new (EphyPasswordManager *manager)
 {
   return EPHY_PASSWORDS_DIALOG (g_object_new (EPHY_TYPE_PASSWORDS_DIALOG,
                                               "password-manager", manager,
-                                              "use-header-bar", TRUE,
                                               NULL));
 }
