@@ -7,11 +7,13 @@ fi
 
 SASSC_OPT="-M -t compact"
 
-: ${GTK_SOURCE_PATH:="../../../gtk+-3"}
+: ${GTK_SOURCE_PATH:="../../../gtk"}
 
 sassc $SASSC_OPT -I${GTK_SOURCE_PATH}/gtk/theme/Adwaita \
 	themes/Adwaita.scss themes/Adwaita.css
 sassc $SASSC_OPT -I${GTK_SOURCE_PATH}/gtk/theme/Adwaita \
 	themes/Adwaita-dark.scss themes/Adwaita-dark.css
+sassc $SASSC_OPT -I${GTK_SOURCE_PATH}/gtk/theme/Adwaita \
+	themes/elementary.scss themes/elementary.css
 sassc $SASSC_OPT -I${GTK_SOURCE_PATH}/gtk/theme/Adwaita \
 	themes/shared.scss themes/shared.css
