@@ -550,6 +550,22 @@ window_cmd_show_shortcuts (GSimpleAction *action,
     builder = gtk_builder_new_from_resource ("/org/gnome/epiphany/gtk/shortcuts-dialog.ui");
     shortcuts_window = GTK_WIDGET (gtk_builder_get_object (builder, "shortcuts-dialog"));
 
+    if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL) {
+      GtkShortcutsShortcut *shortcut;
+
+      shortcut = GTK_SHORTCUTS_SHORTCUT (gtk_builder_get_object (builder, "go-back-shortcut"));
+      g_object_set (shortcut, "accelerator", "<Alt>Right", NULL);
+
+      shortcut = GTK_SHORTCUTS_SHORTCUT (gtk_builder_get_object (builder, "go-forward-shortcut"));
+      g_object_set (shortcut, "accelerator", "<Alt>Left", NULL);
+
+      shortcut = GTK_SHORTCUTS_SHORTCUT (gtk_builder_get_object (builder, "go-back-gesture"));
+      g_object_set (shortcut, "shortcut-type", GTK_SHORTCUT_GESTURE_TWO_FINGER_SWIPE_LEFT, NULL);
+
+      shortcut = GTK_SHORTCUTS_SHORTCUT (gtk_builder_get_object (builder, "go-forward-gesture"));
+      g_object_set (shortcut, "shortcut-type", GTK_SHORTCUT_GESTURE_TWO_FINGER_SWIPE_RIGHT, NULL);
+    }
+
     g_signal_connect (shortcuts_window,
                       "destroy",
                       G_CALLBACK (gtk_widget_destroyed),
@@ -707,12 +723,6 @@ window_cmd_navigation (GSimpleAction *action,
   EphyWindow *window = EPHY_WINDOW (user_data);
   EphyEmbed *embed;
   WebKitWebView *web_view;
-  EphyLinkFlags flags = ephy_link_flags_from_current_event ();
-
-  if (flags & EPHY_LINK_NEW_TAB) {
-    window_cmd_navigation_new_tab (action, parameter, user_data);
-    return;
-  }
 
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
   g_assert (embed != NULL);
@@ -948,17 +958,9 @@ typedef struct {
   char *icon_href;
   GdkRGBA icon_rgba;
   GCancellable *cancellable;
+  WebKitDownload *download;
 } EphyApplicationDialogData;
 
-static void
-ephy_application_dialog_data_free (EphyApplicationDialogData *data)
-{
-  g_cancellable_cancel (data->cancellable);
-  g_object_unref (data->cancellable);
-  g_free (data->icon_href);
-  g_free (data);
-}
-
 static void
 rounded_rectangle (cairo_t *cr,
                    gdouble  aspect,
@@ -1140,30 +1142,29 @@ download_failed_cb (WebKitDownload            *download,
 static void
 download_icon_and_set_image (EphyApplicationDialogData *data)
 {
-  WebKitDownload *download;
   char *destination, *destination_uri, *tmp_filename;
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
-  download = webkit_web_context_download_uri (ephy_embed_shell_get_web_context (shell),
-                                              data->icon_href);
+  data->download = webkit_web_context_download_uri (ephy_embed_shell_get_web_context (shell),
+                                                    data->icon_href);
   /* We do not want this download to show up in the UI, so let's
    * set 'ephy-download-set' to make Epiphany think this is
    * already there. */
   /* FIXME: it's probably better to just do this in a clean way
    * instead of using this workaround. */
-  g_object_set_data (G_OBJECT (download), "ephy-download-set", GINT_TO_POINTER (TRUE));
+  g_object_set_data (G_OBJECT (data->download), "ephy-download-set", GINT_TO_POINTER (TRUE));
 
   tmp_filename = ephy_file_tmp_filename (".ephy-download-XXXXXX", NULL);
   destination = g_build_filename (ephy_file_tmp_dir (), tmp_filename, NULL);
   destination_uri = g_filename_to_uri (destination, NULL, NULL);
-  webkit_download_set_destination (download, destination_uri);
+  webkit_download_set_destination (data->download, destination_uri);
   g_free (destination);
   g_free (destination_uri);
   g_free (tmp_filename);
 
-  g_signal_connect (download, "finished",
+  g_signal_connect (data->download, "finished",
                     G_CALLBACK (download_finished_cb), data);
-  g_signal_connect (download, "failed",
+  g_signal_connect (data->download, "failed",
                     G_CALLBACK (download_failed_cb), data);
 }
 
@@ -1361,6 +1362,22 @@ ephy_focus_desktop_app (const char *desktop_file)
   g_bus_get (G_BUS_TYPE_SESSION, NULL, session_bus_ready_cb, g_strdup (desktop_file));
 }
 
+static void
+ephy_application_dialog_data_free (EphyApplicationDialogData *data)
+{
+  if (data->download) {
+    g_signal_handlers_disconnect_by_func (data->download, download_finished_cb, data);
+    g_signal_handlers_disconnect_by_func (data->download, download_failed_cb, data);
+
+    data->download = NULL;
+  }
+
+  g_cancellable_cancel (data->cancellable);
+  g_object_unref (data->cancellable);
+  g_free (data->icon_href);
+  g_free (data);
+}
+
 static void
 dialog_save_as_application_response_cb (GtkDialog                 *dialog,
                                         gint                       response,
